@@ -1,18 +1,18 @@
 # Schedwi
-# Copyright (C) 2011 Herve Quatremain
-# 
+# Copyright (C) 2011-2013 Herve Quatremain
+#
 # This file is part of Schedwi.
-# 
+#
 # Schedwi is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # Schedwi is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -28,12 +28,15 @@ import path
 import path_cal
 import sql_hierarchy
 import status_utils
+import simple_queries_job
+import environments_utils
 from tables.job_main import job_main
 from tables.job_command import job_command
 from tables.job_arguments import job_arguments
 from tables.job_loadenv import job_loadenv
-from tables.job_file_err import job_file_err
 from tables.job_file_out import job_file_out
+from tables.job_file_err import job_file_err
+from tables.job_control_group import job_control_group
 from tables.job_max_duration import job_max_duration
 from tables.job_retries import job_retries
 from tables.job_retries_interval import job_retries_interval
@@ -43,7 +46,6 @@ from tables.job_username import job_username
 from tables.job_host import job_host
 from tables.hosts import hosts
 from tables.constraint_file import constraint_file
-from tables.environments import environments
 from tables.job_environment import job_environment
 from tables.link import link
 from tables.job_main_s import job_main_s
@@ -52,26 +54,28 @@ from tables.job_command_s import job_command_s
 from tables.job_arguments_s import job_arguments_s
 from tables.job_success_return_code_s import job_success_return_code_s
 from tables.job_username_s import job_username_s
-from tables.job_file_err_s import job_file_err_s
 from tables.job_file_out_s import job_file_out_s
+from tables.job_file_err_s import job_file_err_s
+from tables.job_control_group_s import job_control_group_s
 from tables.job_max_duration_s import job_max_duration_s
 from tables.job_retries_s import job_retries_s
 from tables.job_retries_interval_s import job_retries_interval_s
 from tables.job_start_limit_s import job_start_limit_s
 from tables.job_loadenv_s import job_loadenv_s
 from tables.job_environment_s import job_environment_s
-from tables.environments_s import environments_s
 from tables.constraint_file_s import constraint_file_s
 from tables.link_s import link_s
 from help import print_trim
 
+
 def usage():
     """Print a usage message on STDOUT."""
-    print_trim("""Usage: cat [-h] [JOB|JOBSET]...
+    print_trim(_("""Usage: cat [-h] [JOB|JOBSET]...
     Print job or jobset parameters.
     If JOB|JOBSET is not specified, print the parameters of the current jobset.
-    If a parameter name is followed by the `*' character it means that this
-    parameter value is inherited from the parent jobset (see example below).
+    In the output, if a parameter name is followed by the `*' character, it
+    means that this parameter value is inherited from the parent jobset (see
+    example below).
 
     Option:
       -h, --help  display this help.
@@ -90,11 +94,13 @@ def usage():
                  Username * : hive
               Output file * : /dev/null
                Error file * : /dev/null
+      Linux Control Group * : schedwi
              Max duration * : No limit
         Number of retries * : 0
          Interval retries * : 0h05
     Limit past start time * : 24h00
             Load user env * : False
+    Use environment group   : True
         Environment group   : hive-env
         Environment group   : convert-env
           Constraint file   : /data/share/rpt.txt must exist on flower.example.com(2006)
@@ -108,11 +114,12 @@ def usage():
     If the separator between the parameter name and its value is `=' rather
     than `:', it means that the parameter cannot be changed (for already
     planned jobs and jobsets).
-    """)
+    """))
 
 _FIELD_WIDTH = 21
 
-def _print_val(title, hist, fmt=None, val=None, sep=":"):
+
+def _print_val(title, hist, fmt=None, val=None, writable=True):
     """Print a parameter value.
 
     Arguments:
@@ -120,9 +127,11 @@ def _print_val(title, hist, fmt=None, val=None, sep=":"):
     hist -- whether the parameter is inherited or not
     fmt -- format string for the parameter value
     val -- value
-    sep -- separator between name and value
+    writable -- is the parameter can be changed.  If True, the separator
+                between the name and the value is `:'.  Otherwise it's `='
 
     """
+    sep = ':' if writable else '='
     if fmt is not None:
         if val is not None:
             print "%s %c %s" % (title.rjust(_FIELD_WIDTH),
@@ -134,6 +143,7 @@ def _print_val(title, hist, fmt=None, val=None, sep=":"):
         print "%s %c %s" % (title.rjust(_FIELD_WIDTH), ' ' if hist else '*',
                             sep)
 
+
 def cat(sql_session, current_cwd, arguments, workload=None):
     """Print job/jobset parameters.
 
@@ -148,7 +158,7 @@ def cat(sql_session, current_cwd, arguments, workload=None):
     try:
         optlist, args = getopt.getopt(arguments, "h", ["help"])
     except getopt.GetoptError, err:
-        sys.stderr.write("cat: " + str(err) + "\n")
+        sys.stderr.write(_("cat: ") + str(err) + "\n")
         return 1
     for o, a in optlist:
         if o in ("-h", "--help"):
@@ -162,7 +172,7 @@ def cat(sql_session, current_cwd, arguments, workload=None):
     else:
         paths.append(current_cwd)
     if not paths:
-        sys.stderr.write("cat: no such job or jobset\n")
+        sys.stderr.write(_("cat: no such job or jobset\n"))
         return 1
     first = True
     error = False
@@ -180,19 +190,20 @@ def cat(sql_session, current_cwd, arguments, workload=None):
         try:
             job = query.filter_by(id=p.id[-1]).one()
         except:
-            sys.stderr.write("cat: no such job or jobset\n")
+            sys.stderr.write(_("cat: no such job or jobset\n"))
             error = True
             continue
-        _print_val("Name", True, "%s", job.name if job.id > 1 else '/',
-                   ":" if workload is None else "=")
+        _print_val(_("Name"), True, "%s", job.name if job.id > 1 else '/',
+                   True if workload is None else job_main_s.rw_name)
         if workload is not None:
-            _print_val("Current status", True, "%s",
+            _print_val(_("Current status"), True, "%s",
                        status_utils.get_status_string(session, job.id,
                                                       workload))
-        _print_val("Type", True, "%s", "Jobset" if not job.type else "Job",
-                   ":" if workload is None else "=")
-        _print_val("Enabled", True, "%s", job.enabled,
-                   ":" if workload is None else "=")
+        _print_val(_("Type"), True,
+                   "%s", _("Jobset") if not job.type else _("Job"),
+                   True if workload is None else job_main_s.rw_type)
+        _print_val(_("Enabled"), True, "%s", bool(job.enabled),
+                   True if workload is None else job_main_s.rw_enabled)
         sql_session.close_session(session)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
@@ -200,9 +211,10 @@ def cat(sql_session, current_cwd, arguments, workload=None):
                                 workload)
         session = sql_session.open_session()
         host = session.query(hosts).filter_by(id=obj.host_id).one()
-        _print_val("Host", in_object,
-                    "%s (%s/SSL)" if host.sslenable else "%s (%s)",
-                    (host.hostname, host.portnum))
+        _print_val(_("Host"), in_object,
+                   "%s (%s/SSL)" if host.sslenable else "%s (%s)",
+                   (host.hostname, host.portnum),
+                   True if workload is None else job_main_s.rw_job_host)
 
         if job.cal_id == 0:
             for i in range(len(p.id) - 2, -1, -1):
@@ -213,47 +225,46 @@ def cat(sql_session, current_cwd, arguments, workload=None):
                     query = query.filter(job_main_s.workload_date == workload)
                 cal_id, = query.filter_by(id=p.id[i]).one()
                 if cal_id != 0:
-                    _print_val("Calendar", False, "%s",
+                    _print_val(_("Calendar"), False, "%s",
                         path_cal.PathCal(sql_session, id=cal_id,
                                          workload=workload),
-                               ":" if workload is None else "=")
+                        True if workload is None else job_main_s.rw_cal_id)
                     break
             else:
-                _print_val("Calendar", False, "%s", "Everyday",
-                           ":" if workload is None else "=")
+                _print_val(_("Calendar"), False, "%s", _("Everyday"),
+                           True if workload is None else job_main_s.rw_cal_id)
         else:
-            _print_val("Calendar", True, "%s",
-                        path_cal.PathCal(sql_session,id=job.cal_id,
+            _print_val(_("Calendar"), True, "%s",
+                        path_cal.PathCal(sql_session, id=job.cal_id,
                                          workload=workload),
-                       ":" if workload is None else "=")
+                       True if workload is None else job_main_s.rw_cal_id)
 
         if job.start_time < 0:
-            for i in range(len(p.id) - 2, -1, -1):
-                if workload is None:
-                    query = session.query(job_main.start_time)
-                else:
-                    query = session.query(job_main_s.start_time)
-                    query = query.filter(job_main_s.workload_date == workload)
-                start_time, = query.filter_by(id=p.id[i]).one()
-                if start_time >= 0:
-                    _print_val("Start time", False, "%02dh%02d", 
-                                (start_time/100, start_time%100))
-                    break
+            foo, j = sql_hierarchy.get_start_time_id(session, p.id[:-1],
+                                                     workload)
+            if j is not None:
+                start_time = j.start_time
+                _print_val(_("Start time"), False, "%02dh%02d",
+                            (start_time / 100, start_time % 100),
+                    True if workload is None else job_main_s.rw_start_time)
             else:
-                _print_val("Start time", False, "%02dh%02d", (0, 0))
+                _print_val(_("Start time"), False, "%02dh%02d", (0, 0),
+                    True if workload is None else job_main_s.rw_start_time)
         else:
-            _print_val("Start time", True, "%02dh%02d", 
-                     (job.start_time/100, job.start_time%100),
-                     ":" if workload is None else "=")
+            _print_val(_("Start time"), True, "%02dh%02d",
+                     (job.start_time / 100, job.start_time % 100),
+                     True if workload is None else job_main_s.rw_start_time)
         sql_session.close_session(session)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                             job_command if workload is None else job_command_s,
                             workload)
         if obj is not None:
-            _print_val("Command", in_object, "%s", obj.command)
+            _print_val(_("Command"), in_object, "%s", obj.command,
+                       True if workload is None else job_main_s.rw_job_command)
         else:
-            _print_val("Command", True, "%s", "None")
+            _print_val(_("Command"), True, "%s", _("None"),
+                       True if workload is None else job_main_s.rw_job_command)
 
         in_object, lst = sql_hierarchy.get_list(sql_session, p,
                         job_arguments if workload is None else job_arguments_s,
@@ -265,36 +276,53 @@ def cat(sql_session, current_cwd, arguments, workload=None):
             params = list()
             for param in lst:
                 if param.position == -1:
-                    _print_val("Parameters", in_object)
+                    _print_val(_("Parameters"), in_object,
+                     True if workload is None else job_main_s.rw_job_arguments)
                     break
                 params.append('"' + param.argument + '"')
             else:
-                _print_val("Parameters", in_object, "%s", ' '.join(params))
+                _print_val(_("Parameters"), in_object, "%s", ' '.join(params),
+                    True if workload is None else job_main_s.rw_job_arguments)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                         job_success_return_code if workload is None \
                                                 else job_success_return_code_s,
                         workload)
         if obj is not None:
-            _print_val("Success return code", in_object, "%s", obj.success_ret)
+            _print_val(_("Success return code"), in_object, "%s",
+                       obj.success_ret,
+                       True if workload is None \
+                            else job_main_s.rw_job_success_return_code)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_username if workload is None else job_username_s,
                           workload)
         if obj is not None:
-            _print_val("Username", in_object, "%s", obj.username)
+            _print_val(_("Username"), in_object, "%s", obj.username,
+                    True if workload is None else job_main_s.rw_job_username)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_file_out if workload is None else job_file_out_s,
                           workload)
         if obj is not None:
-            _print_val("Output file", in_object, "%s", obj.file_out)
+            _print_val(_("Output file"), in_object, "%s", obj.file_out,
+                    True if workload is None else job_main_s.rw_job_file_out)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_file_err if workload is None else job_file_err_s,
                           workload)
         if obj is not None:
-            _print_val("Error file", in_object, "%s", obj.file_err)
+            _print_val(_("Error file"), in_object, "%s", obj.file_err,
+                    True if workload is None else job_main_s.rw_job_file_err)
+
+        in_object, obj = sql_hierarchy.get(sql_session, p,
+                          job_control_group if workload is None \
+                                            else job_control_group_s,
+                          workload)
+        if obj is not None:
+            _print_val(_("Linux Control Group"), in_object, "%s",
+                       obj.control_group,
+                True if workload is None else job_main_s.rw_job_control_group)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_max_duration if workload is None \
@@ -302,24 +330,31 @@ def cat(sql_session, current_cwd, arguments, workload=None):
                           workload)
         if obj is not None:
             if obj.max_duration == 0:
-                _print_val("Max duration", in_object, "%s", "No limit")
+                _print_val(_("Max duration"), in_object, "%s", _("No limit"),
+                           True if workload is None \
+                                else job_main_s.rw_job_max_duration)
             else:
-                _print_val("Max duration", in_object, "%dh%02d",
-                           (obj.max_duration/60, obj.max_duration%60))
+                _print_val(_("Max duration"), in_object, "%dh%02d",
+                           (obj.max_duration / 60, obj.max_duration % 60),
+                           True if workload is None \
+                                else job_main_s.rw_job_max_duration)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_retries if workload is None else job_retries_s,
                           workload)
         if obj is not None:
-            _print_val("Number of retries", in_object, "%s", obj.retries)
+            _print_val(_("Number of retries"), in_object, "%s", obj.retries,
+                True if workload is None else job_main_s.rw_job_retries)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_retries_interval if workload is None \
                                                else job_retries_interval_s,
                           workload)
         if obj is not None:
-             _print_val("Interval retries", in_object, "%dh%02d",
-                        (obj.retries_interval/60, obj.retries_interval%60))
+            _print_val(_("Interval retries"), in_object, "%dh%02d",
+                       (obj.retries_interval / 60, obj.retries_interval % 60),
+                       True if workload is None \
+                            else job_main_s.rw_job_retries_interval)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_start_limit if workload is None \
@@ -327,19 +362,49 @@ def cat(sql_session, current_cwd, arguments, workload=None):
                           workload)
         if obj is not None:
             if obj.start_limit <= 0:
-                _print_val("Limit past start time", in_object, "%s",
-                           "No limit")
+                _print_val(_("Limit past start time"), in_object, "%s",
+                           _("No limit"),
+                           True if workload is None \
+                                else job_main_s.rw_job_start_limit)
             else:
-                _print_val("Limit past start time", in_object, "%dh%02d",
-                           (obj.start_limit/60, obj.start_limit%60))
+                _print_val(_("Limit past start time"), in_object, "%dh%02d",
+                           (obj.start_limit / 60, obj.start_limit % 60),
+                           True if workload is None \
+                                else job_main_s.rw_job_start_limit)
 
         in_object, obj = sql_hierarchy.get(sql_session, p,
                           job_loadenv if workload is None else job_loadenv_s,
                           workload)
         if obj is not None:
-            _print_val("Load user env", in_object, "%s", obj.loadenv)
+            _print_val(_("Load user env"), in_object, "%s", bool(obj.loadenv),
+                       True if workload is None else job_main_s.rw_job_loadenv)
+
+        _print_val(_("Use environment group"), True, "%s", bool(job.has_env),
+                   True if workload is None else job_main_s.rw_has_env)
 
         session = sql_session.open_session()
+
+        env_list = list()
+        for i in range(len(p.id) - 2, -1, -1):
+            j = simple_queries_job.sql_get_job(session, p.id[i], workload)
+            if not j.has_env:
+                break
+            if workload is None:
+                query = session.query(job_environment)
+                query = query.filter_by(job_id=p.id[i])
+                query = query.order_by(desc(job_environment.position))
+            else:
+                query = session.query(job_environment_s)
+                query = query.filter_by(job_id=p.id[i])
+                query = query.filter(
+                               job_environment_s.workload_date == workload)
+                query = query.order_by(desc(job_environment_s.position))
+            for je in query.all():
+                env_list.insert(0, je.env_id)
+        for env_id in env_list:
+            e = environments_utils.get_env(session, env_id, workload)
+            _print_val(_("Inherited env group"), True, "%s", e.name)
+
         if workload is None:
             query = session.query(job_environment).filter_by(job_id=p.id[-1])
             query = query.order_by(job_environment.position)
@@ -347,63 +412,28 @@ def cat(sql_session, current_cwd, arguments, workload=None):
             query = session.query(job_environment_s).filter_by(job_id=p.id[-1])
             query = query.filter(job_environment_s.workload_date == workload)
             query = query.order_by(job_environment_s.position)
-        job_inheritance = True
         for je in query.all():
-            if je.position < 0:
-                job_inheritance = False
-                break
-            if workload is None:
-                query = session.query(environments)
-            else:
-                query = session.query(environments_s)
-                query = query.filter(environments_s.workload_date == workload)
-            e = query.filter_by(id=je.env_id).one()
-            _print_val("Environment group", True, "%s", e.name)
-
-        if job_inheritance:
-            env_list = list()
-            for i in range(0, len(p.id) - 1):
-                if workload is None:
-                    query = session.query(job_environment)
-                    query = query.filter_by(job_id=p.id[i])
-                    query = query.order_by(job_environment.position)
-                else:
-                    query = session.query(job_environment_s)
-                    query = query.filter_by(job_id=p.id[i])
-                    query = query.filter(
-                                   job_environment_s.workload_date == workload)
-                    query = query.order_by(job_environment_s.position)
-                for je in query.all():
-                    if je.position < 0:
-                        env_list = list()
-                        break
-                    env_list.append(je.env_id)
-            for id in env_list:
-                if workload is None:
-                    query = session.query(environments)
-                else:
-                    query = session.query(environments_s)
-                    query = query.filter(
-                                   environments_s.workload_date == workload)
-                e = query.filter_by(id=id).one()
-                _print_val("Inherited env group", True, "%s", e.name)
+            e = environments_utils.get_env(session, je.env_id, workload)
+            _print_val(_("Environment group"), True, "%s", e.name)
 
         if workload is None:
             query = session.query(constraint_file, hosts)
             query = query.filter_by(job_id=p.id[-1])
-            query = query.filter(constraint_file.host_id==hosts.id)
+            query = query.filter(constraint_file.host_id == hosts.id)
             query = query.order_by(desc(constraint_file.exist))
         else:
             query = session.query(constraint_file_s, hosts)
             query = query.filter(constraint_file_s.workload_date == workload)
             query = query.filter_by(job_id=p.id[-1])
-            query = query.filter(constraint_file_s.host_id==hosts.id)
+            query = query.filter(constraint_file_s.host_id == hosts.id)
             query = query.order_by(desc(constraint_file_s.exist))
-            
+
         for f, h in query.all():
-            _print_val("Constraint file", True, "%s must %sexist on %s(%s)",
-                        (f.filename, '' if f.exist else 'NOT ',
-                         h.hostname, h.portnum))
+            _print_val(_("Constraint file"), True,
+                       _("%s must %sexist on %s(%s)"),
+                        (f.filename, '' if f.exist else _('NOT '),
+                         h.hostname, h.portnum),
+                   True if workload is None else job_main_s.rw_constraint_file)
 
         if workload is None:
             query = session.query(link)
@@ -413,12 +443,12 @@ def cat(sql_session, current_cwd, arguments, workload=None):
         for lnk in query.filter_by(job_id_source=p.id[-1]).all():
             destination = path.id2path(sql_session, lnk.job_id_destination,
                                        workload)
-            _print_val("Link", True, "%s must be %s",
-                      (destination, status_utils.status2string(
+            _print_val(_("Link"), True, _("%s must be %s"),
+                       (destination, status_utils.status2string(
                                              lnk.required_status)),
-                      ":" if workload is None else "=")
+                       True if workload is None else job_main_s.rw_links)
 
         sql_session.close_session(session)
-        _print_val("Description", True, "%s", job.description)
+        _print_val(_("Description"), True, "%s", job.description,
+                   True if workload is None else job_main_s.rw_description)
     return 0 if not error else 1
-
