@@ -1,18 +1,18 @@
 # Schedwi
-# Copyright (C) 2011 Herve Quatremain
-# 
+# Copyright (C) 2011-2013 Herve Quatremain
+#
 # This file is part of Schedwi.
-# 
+#
 # Schedwi is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # Schedwi is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -32,6 +32,7 @@ from tables.job_retries_interval import job_retries_interval
 from tables.job_username import job_username
 from tables.job_file_out import job_file_out
 from tables.job_file_err import job_file_err
+from tables.job_control_group import job_control_group
 from tables.job_loadenv import job_loadenv
 from tables.job_success_return_code import job_success_return_code
 from tables.job_command import job_command
@@ -43,9 +44,10 @@ import path_cal
 import host_utils
 import environments_utils
 
+
 def usage():
     """Print a usage message on STDOUT."""
-    print """Usage: set [OPTION]... JOB|JOBSET
+    print _("""Usage: set [OPTION]... JOB|JOBSET
 Change a job or jobset parameter.
 
 Options:
@@ -81,6 +83,12 @@ Options:
   --stderr=FILE           command error file (stored on the Schedwi agent).
   --delstderr             use the file name defined in the parent
                           jobset instead.
+  --cgroup=NAME           Linux Control Group name to set for the job.  This
+                          name is relative to the controllers mount point.
+                          For instance, for the /cgroup/cpu/schedwi/test mount
+                          point, NAME must be `schedwi/test'.
+  --delcgroup             use the Linux Control Group defined by the parent
+                          jobset instead.
   --duration=MINUTES      maximum number of minutes for the job/jobset to
                           complete once running.
   --delduration           use the value defined in the parent jobset instead.
@@ -98,7 +106,8 @@ Options:
   --addenv=ENV_NAME       add an environment group to the job/jobset.  This
                           option can be specified several times.
   --delenv=ENV_NAME       remove the provided environment group.
-  --noenv                 do not use any environment group for this job/jobset.
+  --enableenv             enable the use of environment groups.
+  --disableenv            do not use any environment group for this job/jobset.
   --addfile=HOSTNAME[:PORT],True|False,FILE     add a file to check before
                           starting the job/jobset.  The value is composed of
                           three fields separated by a comma.  The first one is
@@ -109,7 +118,8 @@ Options:
   --delfile=HOSTNAME[:PORT],FILE    remove the provided file.
   --description=TEXT      a description.
   -h, --help              display this help.
-    """
+    """)
+
 
 def parse_args(argv):
     """Parse the argument list.
@@ -119,7 +129,7 @@ def parse_args(argv):
 
     Exception:
     getopt.getopterror -- option error (unknow option or missing parameter)
-    
+
     """
     optlist, args = getopt.getopt(argv, "h",
             [
@@ -143,6 +153,8 @@ def parse_args(argv):
                 "delstdout",
                 "stderr=",
                 "delstderr",
+                "cgroup=",
+                "delcgroup",
                 "duration=",
                 "delduration",
                 "retries=",
@@ -155,7 +167,8 @@ def parse_args(argv):
                 "deluser-env",
                 "addenv=",
                 "delenv=",
-                "noenv",
+                "enableenv",
+                "disableenv",
                 "description=",
                 "addfile=",
                 "delfile=",
@@ -163,6 +176,7 @@ def parse_args(argv):
             ])
     return (optlist, args)
 
+
 def set(session, current_cwd, current_cwd_cal, job, opts):
     """Change jobs and jobsets parameters.
 
@@ -180,7 +194,7 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
     envs = list()
     for o, a in opts:
         if o == "--enable":
-            if a[0] in ('Y', 'y', '1', 't', 'T'):
+            if a[0] in (_('Y'), _('y'), '1', _('t'), _('T')):
                 job.enabled = 1
             else:
                 job.enabled = 0
@@ -188,10 +202,11 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             try:
                 host = host_utils.name2host(session, a)
             except NoResultFound:
-                sys.stderr.write("No such host (--host=%s)\n" % a)
+                sys.stderr.write(_("No such host (--host=%s)\n") % a)
                 return 1
             except MultipleResultsFound:
-                sys.stderr.write("More than one host matches the provided name (--host=%s)\n" % a)
+                sys.stderr.write(_("More than one host match the provided \
+name (--host=%s)\n") % a)
                 return 1
             job.job_host = job_host(host.id)
 
@@ -201,7 +216,7 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
         elif o == "--calendar":
             c = path_cal.PathCal(session, path=a, cwd=current_cwd_cal)
             if not c:
-                sys.stderr.write("Unknown calendar %s (--calendar)\n" % a)  
+                sys.stderr.write(_("Unknown calendar %s (--calendar)\n") % a)
                 return 1
             job.cal_id = c.id[-1]
 
@@ -217,7 +232,8 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
                 else:
                     break
             else:
-                sys.stderr.write("Unknown time format (--start).  Should be `H:M' or `H:M AM/PM'\n")  
+                sys.stderr.write(_("Unknown time format (--start).  Should be \
+`H:M' or `H:M AM/PM'\n"))
                 return 1
             job.start_time = tstr.tm_hour * 100 + tstr.tm_min
 
@@ -234,12 +250,12 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             arguments.append(job_arguments(len(arguments) + 1, a))
 
         elif o == "--delparameter":
-             arguments = list()
-             job.job_arguments = [ ]
+            arguments = list()
+            job.job_arguments = []
 
         elif o == "--noparameter":
-             arguments = list()
-             job.job_arguments = [ job_arguments(-1, "") ]
+            arguments = list()
+            job.job_arguments = [job_arguments(-1, "")]
 
         elif o == "--return":
             try:
@@ -248,7 +264,7 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
                     raise ValueError
             except:
                 sys.stderr.write(
-                    "Wrong value for --return.  Must be between 0 and 255\n")  
+                  _("Wrong value for --return.  Must be between 0 and 255\n"))
                 return 1
             job.job_success_return_code = job_success_return_code(v)
 
@@ -266,13 +282,18 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             job.job_file_err = job_file_err(a)
         elif o == "--delstderr":
             del job.job_file_err
+        elif o == "--cgroup":
+            job.job_control_group = job_control_group(a)
+        elif o == "--delcgroup":
+            del job.job_control_group
         elif o == "--duration":
             try:
                 v = int(a)
                 if v < 0:
                     raise ValueError
             except:
-                sys.stderr.write("Wrong value for --duration.  Must be a number of minutes (0 means no limit)\n")  
+                sys.stderr.write(_("Wrong value for --duration.  Must be a \
+number of minutes (0 means no limit)\n"))
                 return 1
             job.job_max_duration = job_max_duration(v)
 
@@ -285,7 +306,7 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
                 if v < 0:
                     raise ValueError
             except:
-                sys.stderr.write("Wrong value for --retries\n")  
+                sys.stderr.write(_("Wrong value for --retries\n"))
                 return 1
             job.job_retries = job_retries(v)
 
@@ -298,7 +319,8 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
                 if v < 0:
                     raise ValueError
             except:
-                sys.stderr.write("Wrong value for --retries-int.  Must be the number of minutes between retries\n")
+                sys.stderr.write(_("Wrong value for --retries-int.  Must be \
+the number of minutes between retries\n"))
                 return 1
             job.job_retries_interval = job_retries_interval(v)
 
@@ -320,7 +342,9 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
                     if v < 0:
                         raise ValueError
                 except:
-                    sys.stderr.write("Wrong value for --limit.  Must be the number of minutes allowed for the job to complete (0 means no limit)\n")  
+                    sys.stderr.write(_("Wrong value for --limit.  Must be the \
+number of minutes for the job to be started past its start time \
+(0 means no limit)\n"))
                     return 1
             job.job_start_limit = job_start_limit(v)
 
@@ -328,7 +352,7 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             del job.job_start_limit
 
         elif o == "--user-env":
-            if a[0] in ('Y', 'y', '1', 't', 'T'):
+            if a[0] in (_('Y'), _('y'), '1', _('t'), _('T')):
                 job.job_loadenv = job_loadenv(1)
             else:
                 job.job_loadenv = job_loadenv(0)
@@ -339,7 +363,7 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             try:
                 env = environments_utils.name2env(session, a)
             except NoResultFound:
-                sys.stderr.write("No such environment %s (--addenv)\n" % a)
+                sys.stderr.write(_("No such environment %s (--addenv)\n") % a)
                 return 1
             try:
                 envs.remove(env.id)
@@ -351,43 +375,48 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             try:
                 env = environments_utils.name2env(session, a)
             except NoResultFound:
-                sys.stderr.write("No such environment %s (--delenv)\n" % a)
+                sys.stderr.write(_("No such environment %s (--delenv)\n") % a)
                 return 1
             for e in job.job_environment:
                 if env.id == e.env_id:
                     job.job_environment.remove(e)
                     break
-            envs = filter(lambda id: id!=env.id, envs)
+            envs = filter(lambda id: id != env.id, envs)
 
-        elif o == "--noenv":
-            envs = list()
-            job.job_environment = [ job_environment(0, -1) ]
+        elif o == "--enableenv":
+            job.has_env = 1
+
+        elif o == "--disableenv":
+            job.has_env = 0
 
         elif o == "--description":
-            job.description = a
+            job.description = a.decode('utf-8')
 
         elif o == "--addfile":
             try:
                 h, exist, filename = a.split(',', 2)
             except:
-                sys.stderr.write("Wrong format for --addfile.  Must be <host>,<0|1>,<file>\n")
+                sys.stderr.write(_("Wrong format for --addfile.  Must be \
+<host>,<0|1>,<file>\n"))
                 return 1
             try:
                 host = host_utils.name2host(session, h)
             except NoResultFound:
-                sys.stderr.write("No such host %s (--addfile=%s)\n" % (h, a))
+                sys.stderr.write(_("No such host %s (--addfile=%s)\n")
+                                 % (h, a))
                 return 1
             except MultipleResultsFound:
-                sys.stderr.write("More than one host matches the provided name %s (--addfile=%s)\n" % (h, a))
+                sys.stderr.write(_("More than one host matches the provided \
+name %s (--addfile=%s)\n") % (h, a))
                 return 1
             try:
                 constraint_files.remove(constraint_file(host.id, filename, 1))
             except:
                 pass
-            if exist[0] in ('Y', 'y', '1', 't', 'T'):
+            if exist[0] in (_('Y'), _('y'), '1', _('t'), _('T')):
                 constraint_files.append(constraint_file(host.id, filename, 1))
             else:
-                constraint_files.append(constraint_file(host.id, filename, 0)) 
+                constraint_files.append(constraint_file(host.id, filename, 0))
 
         elif o == "--delfile":
             try:
@@ -397,18 +426,21 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
                     h, filename = a.split(',', 1)
                 except:
                     sys.stderr.write(
-                        "Wrong format for --delfile.  Must be <host>,<file>\n")
+                    _("Wrong format for --delfile.  Must be <host>,<file>\n"))
                     return 1
             try:
                 host = host_utils.name2host(session, h)
             except NoResultFound:
-                sys.stderr.write("No such host %s (--delfile=%s)\n" % (h, a))
+                sys.stderr.write(_("No such host %s (--delfile=%s)\n")
+                                 % (h, a))
                 return 1
             except MultipleResultsFound:
-                sys.stderr.write("More than one host matches the provided name %s (--delfile=%s)\n" % (h, a))
+                sys.stderr.write(_("More than one host match the provided \
+name %s (--delfile=%s)\n") % (h, a))
                 return 1
             try:
-                job.constraint_file.remove(constraint_file(host.id, filename, 1))
+                job.constraint_file.remove(constraint_file(host.id,
+                                                           filename, 1))
             except:
                 pass
             try:
@@ -431,8 +463,6 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
         new_envs = list()
         last_position = 0
         for e in job.job_environment:
-            if e.position < 0: 
-                continue
             try:
                 envs.index(e.env_id)
             except:
@@ -442,9 +472,8 @@ def set(session, current_cwd, current_cwd_cal, job, opts):
             if e.position > last_position:
                 last_position = e.position
             new_envs.append(e)
-        for env_id in envs: 
+        for env_id in envs:
             last_position += 1
             new_envs.append(job_environment(env_id, last_position))
         job.job_environment = new_envs
     return 0
-
