@@ -1,18 +1,18 @@
 # Schedwi
-# Copyright (C) 2011 Herve Quatremain
-# 
+# Copyright (C) 2011-2013 Herve Quatremain
+#
 # This file is part of Schedwi.
-# 
+#
 # Schedwi is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # Schedwi is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -33,6 +33,7 @@ from tables.job_retries_interval_s import job_retries_interval_s
 from tables.job_username_s import job_username_s
 from tables.job_file_out_s import job_file_out_s
 from tables.job_file_err_s import job_file_err_s
+from tables.job_control_group_s import job_control_group_s
 from tables.job_loadenv_s import job_loadenv_s
 from tables.job_success_return_code_s import job_success_return_code_s
 from tables.job_command_s import job_command_s
@@ -47,11 +48,11 @@ import host_utils
 import environments_utils
 import status_utils
 import parse_config
-import init
+
 
 def usage():
     """Print a usage message on STDOUT."""
-    print """Usage: set [OPTION]... JOB|JOBSET
+    print _("""Usage: set [OPTION]... JOB|JOBSET
 Change a job or jobset parameter.
 
 Options:
@@ -83,6 +84,12 @@ Options:
   --stderr=FILE           command error file (stored on the Schedwi agent).
   --delstderr             use the file name defined in the parent
                           jobset instead.
+  --cgroup=NAME           Linux Control Group name to set for the job.  This
+                          name is relative to the controllers mount point.
+                          For instance, for the /cgroup/cpu/schedwi/test mount
+                          point, NAME must be `schedwi/test'.
+  --delcgroup             use the Linux Control Group defined by the parent
+                          jobset instead.
   --duration=MINUTES      maximum number of minutes for the job/jobset to
                           complete once running.
   --delduration           use the value defined in the parent jobset instead.
@@ -93,7 +100,6 @@ Options:
   --limit=MINUTES         number of minutes for the job to be started past its
                           start time (number of minutes or HH:MM)
   --dellimit              use the value defined in the parent jobset instead.
-
   --user-env=True|False   load (True) or not (False) the user account
                           environment (~/.profile or ~/.bashrc for instance) on
                           the remote host before running the job command.
@@ -101,7 +107,8 @@ Options:
   --addenv=ENV_NAME       add an environment group to the job/jobset.  This
                           option can be specified several times.
   --delenv=ENV_NAME       remove the provided environment group.
-  --noenv                 do not use any environment group for this job/jobset.
+  --enableenv             enable the use of environment groups.
+  --disableenv            do not use any environment group for this job/jobset.
   --addfile=HOSTNAME[:PORT],True|False,FILE     add a file to check before
                           starting the job/jobset.  The value is composed of
                           three fields separated by a comma.  The first one is
@@ -120,7 +127,8 @@ Options:
                           command.
   --description=TEXT      a description.
   -h, --help              display this help.
-    """
+    """)
+
 
 def parse_args(argv):
     """Parse the argument list.
@@ -130,7 +138,7 @@ def parse_args(argv):
 
     Exception:
     getopt.getopterror -- option error (unknow option or missing parameter)
-    
+
     """
     optlist, args = getopt.getopt(argv, "h",
             [
@@ -151,6 +159,8 @@ def parse_args(argv):
                 "delstdout",
                 "stderr=",
                 "delstderr",
+                "cgroup=",
+                "delcgroup",
                 "duration=",
                 "delduration",
                 "retries=",
@@ -163,7 +173,8 @@ def parse_args(argv):
                 "deluser-env",
                 "addenv=",
                 "delenv=",
-                "noenv",
+                "enableenv",
+                "disableenv",
                 "description=",
                 "addfile=",
                 "delfile=",
@@ -172,6 +183,7 @@ def parse_args(argv):
             ])
     return (optlist, args)
 
+
 def set(session, job, opts, workload):
     """Change jobs and jobsets parameters.
 
@@ -192,10 +204,11 @@ def set(session, job, opts, workload):
             try:
                 host = host_utils.name2host(session, a)
             except NoResultFound:
-                sys.stderr.write("No such host (--host=%s)\n" % a)
+                sys.stderr.write(_("No such host (--host=%s)\n") % a)
                 return 1
             except MultipleResultsFound:
-                sys.stderr.write("More than one host matches the provided name (--host=%s)\n" % a)
+                sys.stderr.write(_("More than one host match the provided \
+name (--host=%s)\n") % a)
                 return 1
             job.job_host = job_host_s(host.id, workload)
 
@@ -211,7 +224,8 @@ def set(session, job, opts, workload):
                 else:
                     break
             else:
-                sys.stderr.write("Unknown time format (--start).  Should be `H:M' or `H:M AM/PM'\n")  
+                sys.stderr.write(_("Unknown time format (--start).  Should be \
+`H:M' or `H:M AM/PM'\n"))
                 return 1
             job.start_time = tstr.tm_hour * 100 + tstr.tm_min
 
@@ -228,12 +242,12 @@ def set(session, job, opts, workload):
             arguments.append(job_arguments_s(len(arguments) + 1, a, workload))
 
         elif o == "--delparameter":
-             arguments = list()
-             job.job_arguments = [ ]
+            arguments = list()
+            job.job_arguments = []
 
         elif o == "--noparameter":
-             arguments = list()
-             job.job_arguments = [ job_arguments_s(-1, "", workload) ]
+            arguments = list()
+            job.job_arguments = [job_arguments_s(-1, "", workload)]
 
         elif o == "--return":
             try:
@@ -242,9 +256,10 @@ def set(session, job, opts, workload):
                     raise ValueError
             except:
                 sys.stderr.write(
-                    "Wrong value for --return.  Must be between 0 and 255\n")  
+                  _("Wrong value for --return.  Must be between 0 and 255\n"))
                 return 1
-            job.job_success_return_code = job_success_return_code_s(v, workload)
+            job.job_success_return_code = job_success_return_code_s(v,
+                                                                    workload)
 
         elif o == "--delreturn":
             del job.job_success_return_code
@@ -260,13 +275,18 @@ def set(session, job, opts, workload):
             job.job_file_err = job_file_err_s(a, workload)
         elif o == "--delstderr":
             del job.job_file_err
+        elif o == "--cgroup":
+            job.job_control_group = job_control_group_s(a, workload)
+        elif o == "--delstderr":
+            del job.job_control_group
         elif o == "--duration":
             try:
                 v = int(a)
                 if v < 0:
                     raise ValueError
             except:
-                sys.stderr.write("Wrong value for --duration.  Must be a number of minutes (0 means no limit)\n")  
+                sys.stderr.write(_("Wrong value for --duration.  Must be a \
+number of minutes (0 means no limit)\n"))
                 return 1
             job.job_max_duration = job_max_duration_s(v, workload)
 
@@ -279,7 +299,7 @@ def set(session, job, opts, workload):
                 if v < 0:
                     raise ValueError
             except:
-                sys.stderr.write("Wrong value for --retries\n")  
+                sys.stderr.write(_("Wrong value for --retries\n"))
                 return 1
             job.job_retries = job_retries_s(v, workload)
 
@@ -292,7 +312,8 @@ def set(session, job, opts, workload):
                 if v < 0:
                     raise ValueError
             except:
-                sys.stderr.write("Wrong value for --retries-int.  Must be the number of minutes between retries\n")
+                sys.stderr.write(_("Wrong value for --retries-int.  Must be \
+the number of minutes between retries\n"))
                 return 1
             job.job_retries_interval = job_retries_interval_s(v, workload)
 
@@ -314,7 +335,8 @@ def set(session, job, opts, workload):
                     if v < 0:
                         raise ValueError
                 except:
-                    sys.stderr.write("Wrong value for --limit.  Must be the number of minutes allowed for the job to complete (0 means no limit)\n")  
+                    sys.stderr.write(_("Wrong value for --limit.  Must be \
+the number of minutes allowed for the job to complete (0 means no limit)\n"))
                     return 1
             job.job_start_limit = job_start_limit_s(v, workload)
 
@@ -322,7 +344,7 @@ def set(session, job, opts, workload):
             del job.job_start_limit
 
         elif o == "--user-env":
-            if a[0] in ('Y', 'y', '1', 't', 'T'):
+            if a[0] in (_('Y'), _('y'), '1', _('t'), _('T')):
                 job.job_loadenv = job_loadenv_s(1, workload)
             else:
                 job.job_loadenv = job_loadenv_s(0, workload)
@@ -333,7 +355,7 @@ def set(session, job, opts, workload):
             try:
                 env = environments_utils.name2env(session, a, workload)
             except NoResultFound:
-                sys.stderr.write("No such environment %s (--addenv)\n" % a)
+                sys.stderr.write(_("No such environment %s (--addenv)\n") % a)
                 return 1
             try:
                 envs.remove(env.id)
@@ -345,43 +367,48 @@ def set(session, job, opts, workload):
             try:
                 env = environments_utils.name2env(session, a, workload)
             except NoResultFound:
-                sys.stderr.write("No such environment %s (--delenv)\n" % a)
+                sys.stderr.write(_("No such environment %s (--delenv)\n") % a)
                 return 1
             for e in job.job_environment:
                 if env.id == e.env_id:
                     job.job_environment.remove(e)
                     break
-            envs = filter(lambda id: id!=env.id, envs)
+            envs = filter(lambda id: id != env.id, envs)
 
-        elif o == "--noenv":
-            envs = list()
-            job.job_environment = [ job_environment_s(0, -1, workload) ]
+        elif o == "--enableenv":
+            job.has_env = 1
+
+        elif o == "--disableenv":
+            job.has_env = 0
 
         elif o == "--addfile":
             try:
                 h, exist, filename = a.split(',', 2)
             except:
-                sys.stderr.write("Wrong format for --addfile.  Must be <host>,<0|1>,<file>\n")
+                sys.stderr.write(_("Wrong format for --addfile.  Must be \
+<host>,<0|1>,<file>\n"))
                 return 1
             try:
                 host = host_utils.name2host(session, h)
             except NoResultFound:
-                sys.stderr.write("No such host %s (--addfile=%s)\n" % (h, a))
+                sys.stderr.write(_("No such host %s (--addfile=%s)\n")
+                                 % (h, a))
                 return 1
             except MultipleResultsFound:
-                sys.stderr.write("More than one host matches the provided name %s (--addfile=%s)\n" % (h, a))
+                sys.stderr.write(_("More than one host matches the provided \
+name %s (--addfile=%s)\n") % (h, a))
                 return 1
             try:
                 constraint_files.remove(constraint_file_s(host.id, filename,
                                                           1, workload))
             except:
                 pass
-            if exist[0] in ('Y', 'y', '1', 't', 'T'):
+            if exist[0] in (_('Y'), _('y'), '1', _('t'), _('T')):
                 constraint_files.append(constraint_file_s(host.id, filename,
                                                           1, workload))
             else:
                 constraint_files.append(constraint_file_s(host.id, filename,
-                                                          0, workload)) 
+                                                          0, workload))
 
         elif o == "--delfile":
             try:
@@ -391,15 +418,17 @@ def set(session, job, opts, workload):
                     h, filename = a.split(',', 1)
                 except:
                     sys.stderr.write(
-                        "Wrong format for --delfile.  Must be <host>,<file>\n")
+                    _("Wrong format for --delfile.  Must be <host>,<file>\n"))
                     return 1
             try:
                 host = host_utils.name2host(session, h)
             except NoResultFound:
-                sys.stderr.write("No such host %s (--delfile=%s)\n" % (h, a))
+                sys.stderr.write(_("No such host %s (--delfile=%s)\n")
+                                 % (h, a))
                 return 1
             except MultipleResultsFound:
-                sys.stderr.write("More than one host matches the provided name %s (--delfile=%s)\n" % (h, a))
+                sys.stderr.write(_("More than one host match the provided \
+name %s (--delfile=%s)\n") % (h, a))
                 return 1
             try:
                 job.constraint_file.remove(constraint_file_s(host.id, filename,
@@ -415,11 +444,11 @@ def set(session, job, opts, workload):
         elif o == "--status":
             status_code = status_utils.string2status(a)
             if not status_code:
-                sys.stderr.write("Unknown status %s (--status)\n" % a)
+                sys.stderr.write(_("Unknown status %s (--status)\n") % a)
                 return 1
 
         elif o == "--description":
-            job.description = a
+            job.description = a.decode('utf-8')
 
         elif o in ("-h", "--help"):
             usage()
@@ -437,8 +466,6 @@ def set(session, job, opts, workload):
         new_envs = list()
         last_position = 0
         for e in job.job_environment:
-            if e.position < 0: 
-                continue
             try:
                 envs.index(e.env_id)
             except:
@@ -448,7 +475,7 @@ def set(session, job, opts, workload):
             if e.position > last_position:
                 last_position = e.position
             new_envs.append(e)
-        for env_id in envs: 
+        for env_id in envs:
             last_position += 1
             new_envs.append(job_environment_s(env_id, last_position, workload))
         job.job_environment = new_envs
@@ -472,8 +499,8 @@ def set(session, job, opts, workload):
                     c = commands(job.id, parse_config.DBI_USER, 0, 1,
                                  status_code, workload)
                     session.add(c)
-                    print "It may take a minute for the new status to be applied"
+                    print _("It may take a minute for the new status to \
+be applied")
                 else:
-                    print "This is already the current status"
+                    print _("This is already the current status")
     return 0
-
