@@ -1,26 +1,27 @@
 # Schedwi
-# Copyright (C) 2011 Herve Quatremain
-# 
+# Copyright (C) 2011, 2012 Herve Quatremain
+#
 # This file is part of Schedwi.
-# 
+#
 # Schedwi is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 3 of the License, or
 # (at your option) any later version.
-# 
+#
 # Schedwi is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from sqlalchemy import Column, Integer, Boolean, ForeignKey
-from sqlalchemy.schema import ForeignKeyConstraint
+from sqlalchemy.schema import Column, ForeignKeyConstraint
+from sqlalchemy.types import Integer
 
 from base import Base
 
+
 class job_loadenv_s(Base):
     __tablename__ = 'job_loadenv_s'
     __table_args__ = (
@@ -28,8 +29,8 @@ class job_loadenv_s(Base):
                                 ['job_main_s.id', 'job_main_s.workload_date']),
             {})
 
-    job_id  = Column(Integer, primary_key=True)
-    loadenv = Column(Boolean)
+    job_id = Column(Integer, primary_key=True)
+    loadenv = Column(Integer)
     workload_date = Column(Integer, primary_key=True)
 
     def __init__(self, loadenv, workload_date):
