@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -40,7 +40,7 @@
  * Error callback function for the sql_cal_only_list() function
  */
 static void
-sql_error_logger (void *data, const char *msg, unsigned int err_code)
+sql_error_logger (void *data, const char *msg, int err_code)
 {
     if (msg != NULL) {
 	lwc_writeLog (LOG_ERR, msg);
@@ -69,110 +69,117 @@ static int
 get_calendar_list (	calendar_t_ptr *list, unsigned int *nmemb,
 			schedwi_date workload_date)
 {
-	lwc_LL *cals;
-	char **row;
-	unsigned int ret, i;
-	calendar_t_ptr ptr;
-	cal_errcode_t ret1, ret2, ret3;
-	unsigned short int year;
+   lwc_LL *cals;
+   row_item_t *row;
+   unsigned int i;
+   int ret;
+   calendar_t_ptr ptr;
+   cal_errcode_t ret1, ret2, ret3;
+   unsigned short int year;
+   unsigned long long int id;
+   char *name, *formula;
 
 #if HAVE_ASSERT_H
-	assert (list != NULL);
+   assert (list != NULL);
 #endif
 
-	/* Retrieve all the calendars */
-	ret = sql_cal_only_list (	schedwi_date_to_int (workload_date),
-					&cals, sql_error_logger, NULL);
-	if (ret != 0) {
-		return (ret == 1) ? -1 : -2;
-	}
+   /* Retrieve all the calendars */
+   ret = sql_cal_only_list (	schedwi_date_to_int (workload_date), &cals,
+				sql_error_logger, NULL);
+   if (ret != 0) {
+      return ret;
+   }
 
-	/* Allocate the array */
-	ptr = (calendar_t_ptr) malloc (   sizeof (calendar_t)
+   /* Allocate the array */
+   ptr = (calendar_t_ptr) malloc (	  sizeof (calendar_t)
 					* (lwc_getNumNode (cals) + 1));
-	if (ptr == NULL) {
-		lwc_delLL (cals, (void (*)(const void *)) sql_free_row);
-		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-		return -1;
-	}
+   if (ptr == NULL) {
+      lwc_delLL (cals, (void (*)(const void *)) sql_free_row);
+      lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+      return -1;
+   }
 
-	/* Copy the list in the array */
-	year = workload_date.year;
-	i = 0;
-	while ((row = (char **) lwc_delStartLL (cals)) != NULL) {
-		/*
-		 * row[0] --> ID
-		 * row[1] --> Name
-		 * row[2] --> Formula
-		 */
-		ret1 = str2cal (&(ptr[i].calendar[0]), row[2], year - 1, NULL);
-		ret2 = str2cal (&(ptr[i].calendar[1]), row[2], year, NULL);
-		ret3 = str2cal (&(ptr[i].calendar[2]), row[2], year + 1, NULL);
-		switch ((ret1 == CAL_NOERROR) ? 
-				((ret2 == CAL_NOERROR) ? ret3 : ret2) : ret1)
-		{
-			case CAL_NOERROR:
-				ptr[i++].id = strtoull (row[0], NULL, 0);
-				break;
+   /* Copy the list in the array */
+   year = workload_date.year;
+   i = 0;
+   while ((row = (row_item_t *) lwc_delStartLL (cals)) != NULL) {
+      /*
+       * row[0] --> ID
+       * row[1] --> Name
+       * row[2] --> Formula
+       */
+      id      = (unsigned long long int)sql_row_item2ll (&(row[0]));
+      name    = row[1].value_string;
+      formula = row[2].value_string;
 
-			case CAL_MALLOC:
-				lwc_writeLog (LOG_ERR,
-_("Memory allocation error when trying to parse the calendar `%s' (ID: %s)"),
-					row[1], row[0]);
-				break;
+      ret1 = str2cal (&(ptr[i].calendar[0]), formula, year - 1, NULL);
+      ret2 = str2cal (&(ptr[i].calendar[1]), formula, year, NULL);
+      ret3 = str2cal (&(ptr[i].calendar[2]), formula, year + 1, NULL);
+      switch	((ret1 == CAL_NOERROR) ?
+		((ret2 == CAL_NOERROR) ? ret3 : ret2) : ret1)
+      {
+	case CAL_NOERROR:
+	    ptr[i++].id = id;
+	    break;
 
-			case CAL_EMPTYFIELD:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): a required field is empty"),
-					row[1], row[0]);
-				break;
+	 case CAL_MALLOC:
+	    lwc_writeLog (LOG_ERR,
+_("Memory allocation error when trying to parse the calendar `%s' (ID: %lld)"),
+	                  name, id);
+	    break;
 
-			case CAL_BADMONTHNAME:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): invalid month name"),
-					row[1], row[0]);
-				break;
+	 case CAL_EMPTYFIELD:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): a required field is empty"),
+	                  name, id);
+	    break;
 
-			case CAL_BADDAYNAME:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): invalid day name"),
-					row[1], row[0]);
-				break;
+	 case CAL_BADMONTHNAME:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): invalid month name"),
+	                  name, id);
+	    break;
 
-			case CAL_MONTHOOR:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): month number must be between 1 and 12"),
-					row[1], row[0]);
-				break;
+	 case CAL_BADDAYNAME:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): invalid day name"),
+	                  name, id);
+	    break;
 
-			case CAL_DAYOOR:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): day number must be between 1 and 31"),
-					row[1], row[0]);
-				break;
+	 case CAL_MONTHOOR:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): month number must be between 1 and 12"),
+	                  name, id);
+	    break;
 
-			case CAL_BADOPTION:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): invalid option"),
-					row[1], row[0]);
-				break;
+	 case CAL_DAYOOR:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): day number must be between 1 and 31"),
+	                  name, id);
+	    break;
 
-			default:
-				lwc_writeLog (LOG_ERR,
-_("Calendar syntax error: calendar `%s' (ID: %s): syntax error"),
-					row[1], row[0]);
-				break;
-		}
-		sql_free_row (row);
-	}
-	lwc_delLL (cals, NULL);
-	ptr[i].id = 0;
+	 case CAL_BADOPTION:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): invalid option"),
+	                  name, id);
+	    break;
 
-	*list = ptr;
-	if (nmemb != NULL) {
-		*nmemb = i;
-	}
-	return 0;
+	 default:
+	    lwc_writeLog (LOG_ERR,
+_("Calendar syntax error: calendar `%s' (ID: %lld): syntax error"),
+	                  name, id);
+	    break;
+      }
+      sql_free_row (row);
+   }
+   lwc_delLL (cals, NULL);
+   ptr[i].id = 0;
+
+   *list = ptr;
+   if (nmemb != NULL) {
+      *nmemb = i;
+   }
+   return 0;
 }
 
 
