@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -80,18 +80,14 @@ command_from_int (int cmd)
  *   NULL in case of memory allocation error
  */
 static command_action_ptr
-command_action_new (	const char *workload_str,
-			const char *job_id_str,
+command_action_new (	unsigned int workload,
+			unsigned long long int job_id,
 			const char *user_details,
 			int cmd,
 			int parameter)
 {
 	command_action_ptr ptr;
 
-#if HAVE_ASSERT_H
-	assert (workload_str != NULL && job_id_str != NULL);
-#endif
-
 	ptr = (command_action_ptr) malloc (sizeof (command_action));
 	if (ptr == NULL) {
 		return NULL;
@@ -109,8 +105,8 @@ command_action_new (	const char *workload_str,
 		ptr->user_details = NULL;
 	}
 
-	schedwi_date_from_string (workload_str, &(ptr->workload));
-	ptr->job_id = strtoull (job_id_str, NULL, 0);
+	schedwi_date_from_int (workload, &(ptr->workload));
+	ptr->job_id = job_id;
 	ptr->cmd = command_from_int (cmd);
 	ptr->parameter = parameter;
 	return ptr;
@@ -154,7 +150,7 @@ command_action_cmp (const void *key, const void *obj)
  */
 static void
 sql_command_update_error_logger (	void *data, const char *msg,
-					unsigned int err_code)
+					int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -178,7 +174,7 @@ command_done (command_action_ptr ptr)
 {
 	char id_str[25];
 	int wl_date;
-	unsigned int ret;
+	int ret;
 
 	if (ptr != NULL) {
 		wl_date = schedwi_date_to_int (ptr->workload);
@@ -205,7 +201,7 @@ command_failed (command_action_ptr ptr)
 {
 	char id_str[25];
 	int wl_date;
-	unsigned int ret;
+	int ret;
 
 	if (ptr != NULL) {
 		wl_date = schedwi_date_to_int (ptr->workload);
@@ -273,7 +269,7 @@ command_purge (char **err_msg)
  */
 static void
 sql_command_get_error_logger (	void *data, const char *msg,
-				unsigned int err_code)
+				int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -296,15 +292,14 @@ sql_command_get_error_logger (	void *data, const char *msg,
  */
 static int
 command_get_list_cb (	void *lst,
-			const char *workload_str, const char *job_id_str,
+			unsigned int workload, unsigned long long int job_id,
 			const char *user_name, int cmd, int parameter)
 {
 	lwc_LL *list = (lwc_LL *) lst;
 	command_action_ptr ptr;
 	void *old;
 
-	ptr = command_action_new (	workload_str, job_id_str, user_name,
-					cmd, parameter);
+	ptr = command_action_new (workload, job_id, user_name, cmd, parameter);
 	if (ptr == NULL) {
 		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 		return -1;
@@ -338,7 +333,7 @@ lwc_LL *
 command_get_list ()
 {
 	lwc_LL *list;
-	unsigned int ret;
+	int ret;
 
 	list = lwc_newLL ();
 	if (list == NULL) {
@@ -354,7 +349,6 @@ command_get_list ()
 		return NULL;
 	}
 
-
 	return list;
 }
 
