@@ -33,7 +33,7 @@ extern "C" {
 
 extern int job_status_set OF((	unsigned long long int id,
 				const char *path,
-				schedwi_date workload_date,
+				int workload_date,
 				job_status_state status,
 				unsigned int status_time,
 				long int duration,
