@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -54,7 +54,7 @@ extern int errno;
  * Error callback function for the sql_status_workload_list() function
  */
 static void
-sql_error_logger (void *data, const char *msg, unsigned int err_code)
+sql_error_logger (void *data, const char *msg, int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -71,7 +71,7 @@ sql_error_logger (void *data, const char *msg, unsigned int err_code)
  */
 static void
 sql_status_delete_error_logger (void *data, const char *msg,
-				unsigned int err_code)
+				int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -88,7 +88,7 @@ sql_status_delete_error_logger (void *data, const char *msg,
  */
 static void
 sql_status_copy_error_logger (	void *data, const char *msg,
-				unsigned int err_code)
+				int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -112,7 +112,7 @@ int
 load_workload_class_from_status_table (load_workload_class_ptr ptr)
 {
 	lwc_LL *rows;
-	char **row;
+	row_item_t *row;
 	schedwi_date workload_date;
 	unsigned short int previous_workload_year;
 	calendar_list_t_ptr calendars;
@@ -131,7 +131,7 @@ load_workload_class_from_status_table (load_workload_class_ptr ptr)
 	previous_workload_year = 0;
 	calendars = NULL;
 	pthread_mutex_lock (&(ptr->mutex));
-	while ((row = (char **)lwc_delStartLL (rows)) != NULL) {
+	while ((row = (row_item_t *)lwc_delStartLL (rows)) != NULL) {
 		/*
 		 * row[0] is the workload date (YYYYMMDD)
 		 */
@@ -140,10 +140,13 @@ load_workload_class_from_status_table (load_workload_class_ptr ptr)
 		 * Convert the date. In case of syntax error a warning message
 		 * is logged and the date is ignored
 		 */
-		if (schedwi_date_from_string (row[0], &workload_date) != 0) {
+		if (schedwi_date_from_int (
+				(unsigned int)sql_row_item2ll (&(row[0])),
+				&workload_date) != 0)
+		{
 			lwc_writeLog (LOG_WARNING,
-		_("Wrong date format `%s' in the job_status database table"),
-					row[0]);
+		_("Wrong date format `%d' in the job_status database table"),
+					(int)sql_row_item2ll (&(row[0])));
 			sql_free_row (row);
 			continue;
 		}
@@ -307,7 +310,7 @@ load_workload_class_new ()
 		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 		return NULL;
 	}
-	
+
 	if (pthread_mutex_init (&(ptr->mutex), NULL) != 0) {
 		lwc_writeLog (  LOG_ERR,
 				_("Internal error: mutex initialization: %s"),
