@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -50,6 +50,8 @@
 #include <job_status_state.h>
 #include <workload_class.h>
 #include <net_parse.h>
+#include <JSON_parser.h>
+
 
 #define WORKLOAD_LEN 8
 
@@ -70,9 +72,11 @@ result_to_status (	schedwi_date workload_date,
 			char **job_err_msg)
 {
 	unsigned int len;
-	char *status_msg, *msg, *err_msg, *success_ret;
+	char *status_msg, *msg, *err_msg;
+	row_item_t success_ret;
 	int ret, d;
 	lwc_LL *hierarchy_list;
+	unsigned long long int job_id;
 
 #if HAVE_ASSERT_H
 	assert (ptr != NULL && new_state != NULL && job_err_msg != NULL);
@@ -109,9 +113,9 @@ result_to_status (	schedwi_date workload_date,
 	 * job is successful or not
 	 */
 	d = schedwi_date_to_int (workload_date);
+	job_id = strtoull (ptr->job_id + WORKLOAD_LEN + 1, NULL, 0);
 	err_msg = NULL;
-	hierarchy_list = get_hierarchy_list (d, ptr->job_id + WORKLOAD_LEN + 1,
-						&err_msg);
+	hierarchy_list = get_hierarchy_list (d, job_id, &err_msg);
 	if (hierarchy_list == NULL) {
 		free (msg);
 		if (err_msg != NULL) {
@@ -129,7 +133,7 @@ result_to_status (	schedwi_date workload_date,
 	err_msg = NULL;
 	if (get_job_parameter (	d, hierarchy_list,
 				"job_success_return_code_s", "success_ret",
-				&success_ret, NULL, &err_msg) != 0)
+				&success_ret, &err_msg) != 0)
 	{
 		free (msg);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
@@ -146,24 +150,38 @@ result_to_status (	schedwi_date workload_date,
 	}
 	lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 
-	if (success_ret == NULL || ptr->exit_code > atoi (success_ret)) {
+	if (	   success_ret.type == RES_END
+		|| ptr->exit_code > sql_row_item2ll (&success_ret))
+	{
 		*new_state = JOB_STATUS_STATE_FAILED;
 	}
 	else {
 		*new_state = JOB_STATUS_STATE_COMPLETED;
 	}
 
-	if (success_ret != NULL) {
-		free (success_ret);
-	}
-
 	*job_err_msg = msg;
 	return 0;
 }
 
 
 /*
- * Parse the client request
+ * JSON callback function
+ */
+static int
+json_char_callback (void* ctx, int type, const JSON_value* value)
+{
+	char *s = (char *)ctx;
+
+	if (type == JSON_T_STRING) {
+		strncpy (s, value->vu.str.value, SCHEDWI_NET_FUNCTION_LENGHT);
+		s[SCHEDWI_NET_FUNCTION_LENGHT] = '\0';
+	}
+	return 1;
+}
+
+
+/*
+ * Parse the agent request
  *
  * Return:
  *   0 --> No error
@@ -173,16 +191,18 @@ int
 net_parse (schedwi_BIO *b, void *obj)
 {
 	load_workload_class_ptr workload_list = (load_workload_class_ptr) obj;
-	char *buff, *err_msg;
-	unsigned int len;
-	int nb_read;
+	char *buff, *err_msg, *s;
+	size_t len;
+	int nb_read, i;
 	char module[SCHEDWI_NET_FUNCTION_LENGHT + 1];
-	lwc_LL *list;
 	result_t *ptr;
 	char workload[WORKLOAD_LEN + 1];
 	schedwi_date workload_date;
 	workload_class_ptr workload_obj;
 	job_status_state new_state;
+	JSON_config config;
+	struct JSON_parser_struct *jc;
+
 
 #if HAVE_ASSERT_H
 	assert (workload_list != NULL);
@@ -203,18 +223,41 @@ net_parse (schedwi_BIO *b, void *obj)
 		return -1;
 	}
 
-	/* Check the module name */
-	if (nb_read < SCHEDWI_NET_FUNCTION_LENGHT) {
+	/* JSON parser initialization */
+	init_JSON_config (&config);
+
+	module[0]                     = '\0';
+	config.depth                  = 20;
+	config.callback               = &json_char_callback;
+	config.allow_comments         = 0;
+	config.handle_floats_manually = 0;
+	config.callback_ctx           = module;
+
+	jc = new_JSON_parser (&config);
+
+	/* Look for the first JSON string which is the module name */
+	for (i = 0; i < nb_read && module[0]  == '\0'; i++) {
+		if (! JSON_parser_char (jc, buff[i])) {
+			if (buff != NULL) {
+				free (buff);
+			}
+			lwc_writeLog (LOG_ERR,
+			_("Wrong network request: JSON syntax error"));
+			delete_JSON_parser (jc);
+			return -1;
+		}
+	}
+	delete_JSON_parser (jc);
+
+	if (module[0] == '\0') {
 		if (buff != NULL) {
 			free (buff);
 		}
 		lwc_writeLog (LOG_ERR,
-			_("Wrong network request: not enough characters"));
+			_("Wrong network request: module name not specified"));
 		return -1;
 	}
 
-	strncpy (module, buff, SCHEDWI_NET_FUNCTION_LENGHT);
-	module[SCHEDWI_NET_FUNCTION_LENGHT] = '\0';
 	if (strcmp (module, "result") != 0) {
 		if (buff != NULL) {
 			free (buff);
@@ -225,18 +268,19 @@ net_parse (schedwi_BIO *b, void *obj)
 		return -1;
 	}
 
-	/* Convert the string to a list of result_t structures */
-	if (str_to_results (	buff + SCHEDWI_NET_FUNCTION_LENGHT,
-				nb_read - SCHEDWI_NET_FUNCTION_LENGHT,
-				&list) != 0)
-	{
+	/* Convert the string to a result_t structure */
+	ptr = NULL;
+	if (str_to_result (buff, nb_read, &ptr) != 0) {
 		if (buff != NULL) {
 			free (buff);
 		}
-		err_msg = _("Memory allocation error");
-		net_write (b, "1", 1);
-		net_write (b, err_msg, schedwi_strlen (err_msg));
-		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+		s = "{ \"success\" : false, \"data\" : \"";
+		net_write (b, s, schedwi_strlen (s));
+		s = _("Memory allocation error");
+		net_write (b, s, schedwi_strlen (s));
+		lwc_writeLog (LOG_CRIT, s);
+		s = "\" }";
+		net_write (b, s, schedwi_strlen (s));
 		return -1;
 	}
 
@@ -244,75 +288,64 @@ net_parse (schedwi_BIO *b, void *obj)
 		free (buff);
 	}
 
-	/* Update the status of the jobs */
-	while ((ptr = (result_t *) lwc_delStartLL (list)) != NULL) {
+	/* Update the status of the job */
 
-		strncpy (workload, ptr->job_id, WORKLOAD_LEN);
-		workload[WORKLOAD_LEN] = '\0';
+	strncpy (workload, ptr->job_id, WORKLOAD_LEN);
+	workload[WORKLOAD_LEN] = '\0';
 
-		if (	   ptr->job_id_len < WORKLOAD_LEN + 2
-			|| (ptr->job_id)[WORKLOAD_LEN] !=
-						SCHEDWI_WORKLOAD_SEPARATOR
-			|| schedwi_date_from_string (	workload,
-							&workload_date) != 0)
-		{
-			lwc_delLL (list,
-				(void (*)(const void *)) destroy_result);
-			err_msg = _("Wrong job ID");
-			net_write (b, "1", 1);
-			net_write (b, err_msg, schedwi_strlen (err_msg));
-			lwc_writeLog (	LOG_ERR,
-					_("Wrong job ID `%s'"), ptr->job_id);
-			destroy_result (ptr);
-			return -1;
-		}
+	if (	   ptr->job_id_len < WORKLOAD_LEN + 2
+		|| (ptr->job_id)[WORKLOAD_LEN] != SCHEDWI_WORKLOAD_SEPARATOR
+		|| schedwi_date_from_string (workload, &workload_date) != 0)
+	{
+		s = "{ \"success\" : false, \"data\" : \"";
+		net_write (b, s, schedwi_strlen (s));
+		s = _("Wrong job ID");
+		net_write (b, s, schedwi_strlen (s));
+		s = "\" }";
+		net_write (b, s, schedwi_strlen (s));
+		lwc_writeLog (LOG_ERR, _("Wrong job ID `%s'"), ptr->job_id);
+		destroy_result (ptr);
+		return -1;
+	}
 
-		/* Convert the received result to a status */
-		if (result_to_status (	workload_date, ptr,
-					&new_state, &err_msg) != 0)
-		{
-			lwc_delLL (list,
-				(void (*)(const void *)) destroy_result);
-			destroy_result (ptr);
-			err_msg = _("Server internal error");
-			net_write (b, "1", 1);
-			net_write (b, err_msg, schedwi_strlen (err_msg));
-			return -1;
-		}
+	/* Convert the received result to a status */
+	if (result_to_status (workload_date, ptr, &new_state, &err_msg) != 0) {
+		destroy_result (ptr);
+		s = "{ \"success\" : false, \"data\" : \"";
+		net_write (b, s, schedwi_strlen (s));
+		s = _("Server internal error");
+		net_write (b, s, schedwi_strlen (s));
+		s = "\" }";
+		net_write (b, s, schedwi_strlen (s));
+		return -1;
+	}
 
-		/* Retrieve the workload tree */
-		workload_obj = load_workload_class_find (workload_list,
-							workload_date);
-		if (workload_obj == NULL) {
-			/* Workload not found, ignore the status */
-			free (err_msg);
-			destroy_result (ptr);
-			continue;
-		}
-
-		/* Update the status of the job */
-		if (workload_class_job_finished (workload_obj,
+	/* Retrieve the workload tree */
+	workload_obj = load_workload_class_find (workload_list, workload_date);
+	if (	   workload_obj != NULL
+		&& workload_class_job_finished (workload_obj,
 				strtoull (	ptr->job_id + WORKLOAD_LEN + 1,
 						NULL, 0),
 				new_state,
 				(long int)ptr->duration,
 				err_msg) != 0)
-		{
-			free (err_msg);
-			lwc_delLL (list,
-				(void (*)(const void *)) destroy_result);
-			destroy_result (ptr);
-			err_msg = _("Server internal error");
-			net_write (b, "1", 1);
-			net_write (b, err_msg, schedwi_strlen (err_msg));
-			return -1;
-		}
+	{
 		free (err_msg);
 		destroy_result (ptr);
+		s = "{ \"success\" : false, \"data\" : \"";
+		net_write (b, s, schedwi_strlen (s));
+		s = _("Server internal error");
+		net_write (b, s, schedwi_strlen (s));
+		s = "\" }";
+		net_write (b, s, schedwi_strlen (s));
+		return -1;
 	}
+	free (err_msg);
+	destroy_result (ptr);
+
+	s = "{ \"success\" : true }";
+	net_write (b, s, schedwi_strlen (s));
 
-	lwc_delLL (list, (void (*)(const void *)) destroy_result);
-	net_write (b, "0", 1);
 	return 0;
 }
 
