@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -21,10 +21,6 @@
 
 #include <schedwi.h>
 
-#if HAVE_SYS_TYPES_H
-#include <sys/types.h>
-#endif
-
 #if STDC_HEADERS
 #include <stdlib.h>
 #include <string.h>
@@ -45,6 +41,29 @@
 #include <netinet/in.h>
 #endif
 
+#if HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+
+#if HAVE_NETDB_H
+#include <netdb.h>
+#endif
+
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#else
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#if HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#endif
+
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
 #if HAVE_ERRNO_H
 #include <errno.h>
 #endif
@@ -52,8 +71,12 @@
 extern int errno;
 #endif
 
-#if HAVE_UNISTD_H
-#include <unistd.h>
+#if HAVE_TCPD_H
+#include <tcpd.h>
+#endif
+
+#if HAVE_SYSLOG_H
+#include <syslog.h>
 #endif
 
 #if HAVE_ASSERT_H
@@ -72,17 +95,24 @@ extern int errno;
 #include <sql_hosts.h>
 #include <net_utils_sock.h>
 #include <net_utils_ssl.h>
+#include <parse_json_result.h>
+#include <addrmatch.h>
 #include <net_utils.h>
 
-#define ENF_OF_REQUEST "\nbYe"
+
+#if HAVE_LIBWRAP
+int allow_severity;
+int deny_severity;
+#endif
 
 
 /*
  * Structure used to exchange parameters between the net_accept() function and
- * the new thread which manages the client connection
- */ 
+ * the new thread which manages the agent connection
+ */
 struct _net_utils_data {
-	struct in_addr in;
+	struct sockaddr_storage in;
+	socklen_t inlen;
 	int (*run)(schedwi_BIO *, void *);
 	void *user_data;
 	int s;
@@ -99,7 +129,7 @@ typedef struct _net_utils_data *net_utils_data_ptr;
  *   NULL in case of memory allocation error.
  */
 static net_utils_data_ptr
-net_utils_data_new (	int s, struct sockaddr_in *in,
+net_utils_data_new (	int s, struct sockaddr_storage *in, socklen_t inlen,
 			int (*run)(schedwi_BIO *, void *), void *user_data)
 {
 	net_utils_data_ptr ptr;
@@ -112,7 +142,8 @@ net_utils_data_new (	int s, struct sockaddr_in *in,
 	if (ptr == NULL) {
 		return NULL;
 	}
-	ptr->in        = in->sin_addr;
+	ptr->in        = *in;
+	ptr->inlen     = inlen;
 	ptr->s         = s;
 	ptr->run       = run;
 	ptr->user_data = user_data;
@@ -141,20 +172,20 @@ net_utils_data_destroy (net_utils_data_ptr ptr)
  */
 static void
 sql_host_certificate_error_logger (	void *data, const char *msg,
-					unsigned int err_code)
+					int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
 	}
 	else {
 		lwc_writeLog (LOG_ERR,
-	_("Database error while trying to retrieve the client parameters"));
+	_("Database error while trying to retrieve the agent parameters"));
 	}
 }
 
 
 /*
- * Thread function used to manage the new client connection
+ * Thread function used to manage the new agent connection
  */
 static void *
 net_utils_server_thread (void *data)
@@ -176,7 +207,8 @@ net_utils_server_thread (void *data)
 	pthread_cleanup_push (thread_clean, NULL);
 
 	/* Retrieve the names associated with the in_addr structure */
-	if (build_client_names_from_addr (ptr->in, &names) != 0) {
+	if (build_client_names_from_addr (&(ptr->in), ptr->inlen, &names) != 0)
+	{
 		net_utils_data_destroy (ptr);
 		pthread_exit (NULL);
 	}
@@ -216,7 +248,6 @@ net_utils_server_thread (void *data)
 	/*
 	 * SSL
 	 */
-
 	in.trusted_cert_ptr = (gnutls_certificate_credentials_t *) malloc (
 				sizeof (gnutls_certificate_credentials));
 	if (in.trusted_cert_ptr == NULL) {
@@ -231,7 +262,8 @@ net_utils_server_thread (void *data)
 	in.use_ssl = 1;
 	in.fd = ptr->s;
 
-	if (net_server_ssl (&(in.session), in.fd, in.trusted_cert_ptr, names,
+	if (net_server_ssl (&(in.session), in.fd, in.trusted_cert_ptr,
+			(const char **)names,
 			client_certificate, len_client_certificate) != 0)
 	{
 		if (client_certificate != NULL) {
@@ -257,10 +289,219 @@ net_utils_server_thread (void *data)
 }
 
 
+/*
+ * Retrieve the next waiting connection from the provided set.
+ * sock_set and nb_set are updated accordingly.
+ *   sock_set --> Set of waiting sockets
+ *   max_set  --> Highest socket number + 1 in the set
+ *   nb_set   --> Number of waiting sockets in the set
+ *   allow_from --> Only accept connections from the allowed clients
+ *
+ * Return :
+ *   The next socket
+ *   -1 in case of error (errno is set and a message is displayed using
+ *      lwc_writeLog())
+ *   -2 No more socket waiting (the set is empty)
+ */
+static int
+net_accept_next (fd_set *sock_set, int max_set, int *nb_set,
+		const char *allow_from,
+		struct sockaddr_storage *out_sockname, socklen_t *out_len)
+{
+	int i, nb, save_errno, s, ret;
+	struct sockaddr_storage sockname;
+	char name[NI_MAXHOST];
+	socklen_t len;
+#if HAVE_LIBWRAP
+	struct request_info req;
+#endif
+
+	nb = *nb_set;
+	for (i = 0 ; i < max_set && nb > 0; i++) {
+		if (FD_ISSET (i, sock_set) == 0) {
+			continue;
+		}
+		nb--;
+
+		do {
+			len = sizeof (sockname);
+			s = accept (i, (struct sockaddr *)&sockname, &len);
+			if (s < 0 && errno != EINTR && errno != EAGAIN) {
+				save_errno = errno;
+				lwc_writeLog (	LOG_CRIT,
+						_("Network error: accept: %s"),
+						strerror (errno));
+				errno = save_errno;
+				return -1;
+			}
+		} while (s < 0);
+
+		FD_CLR (i, sock_set);
+		(*nb_set)--;
+
+		/*
+		 * Only the allowed clients, defined by the configuration
+		 * option ALLOW_FROM are allowed.
+		 * It's always allowed if the variable is not set
+		 */
+		if (allow_from == NULL || allow_from[0] == '\0') {
+			if (out_sockname != NULL) {
+				*out_sockname = sockname;
+			}
+			if (out_len != NULL) {
+				*out_len = len;
+			}
+			return s;
+		}
+
+		ret = getnameinfo (	(struct sockaddr *)&sockname, len,
+					name, sizeof (name), NULL, 0,
+					NI_NUMERICHOST);
+		if (ret != 0) {
+			lwc_writeLog (	LOG_NOTICE,
+					_("getnameinfo: %s"),
+					gai_strerror (ret));
+			net_close_sock (s);
+			continue;
+		}
+
+#if HAVE_LIBWRAP
+		/* Check the network access using the TCP Wrappers */
+		allow_severity = LOG_INFO;
+		deny_severity  = LOG_WARNING;
+
+		request_init (	&req,
+				RQ_DAEMON, SCHEDWI_TCP_WRAPPERS_DAEMON,
+				RQ_FILE, s, 0);
+		fromhost (&req);
+		if (hosts_access (&req) == 0) {
+			lwc_writeLog (	LOG_DEBUG,
+			_("Connection refused by the TCP Wrappers for %s"),
+					name);
+			net_close_sock (s);
+			continue;
+		}
+#endif /* HAVE_LIBWRAP */
+
+		/*
+		 * Check through the internal network access list if the agent
+		 * is allowed.  This list is set through the configuration
+		 * variable ALLOW_FROM.
+		 */
+		ret = addr_match_list (name, allow_from);
+		switch (ret) {
+			case 1:
+				/* Allowed */
+				if (out_sockname != NULL) {
+					*out_sockname = sockname;
+				}
+				if (out_len != NULL) {
+					*out_len = len;
+				}
+				return s;
+			case 0:
+				lwc_writeLog (	LOG_DEBUG,
+						_("Connection refused for %s"),
+						name);
+				break;
+			case -1:
+				lwc_writeLog (	LOG_CRIT,
+						_("Memory allocation error"));
+				break;
+			case -2:
+				/*
+				 * Should not occur (checked at start time
+				 * by check_netlist())
+				 */
+				lwc_writeLog (	LOG_ERR,
+	_("Inconsistent mask length in one of the address in ALLOW_FROM"));
+				break;
+		}
+
+		net_close_sock (s);
+	}
+	return -2;
+}
+
+
+/*
+ * Wait for a network request and return the socket associated with the
+ * accepted connection.  The agent host is check to be sure that it's an
+ * allowed client.
+ *
+ * Return:
+ *   The new socket or
+ *   -1 in case of error (errno is set and a message is displayed using
+ *      lwc_writeLog())
+ */
+int
+net_accept_sock (fd_set *sock_set, int sock_max,
+		const char *allow_from, char ***names)
+{
+	static fd_set waiting_set;
+	static int still_waiting = 0;
+	int s, ret, save_errno;
+	fd_set rfds;
+	struct sockaddr_storage sockname;
+	socklen_t len;
+
+#if HAVE_ASSERT_H
+	assert (sock_set != NULL);
+#endif
+
+	/* First, manage the connections still waiting from a previous call */
+	s = net_accept_next (	&waiting_set, sock_max, &still_waiting,
+				allow_from, &sockname, &len);
+	if (s == -1) {
+		return -1;
+	}
+	if (s >= 0) {
+		if (names != NULL) {
+			build_client_names_from_addr (&sockname, len, names);
+		}
+		return s;
+	}
+
+	/* No more pending connections from a previous call */
+
+	while (1) {
+		rfds = *sock_set;
+		ret = select (sock_max, &rfds, NULL, NULL, NULL);
+		if (ret < 0) {
+			if (errno == EINTR || errno == EAGAIN) {
+				continue;
+			}
+			save_errno = errno;
+			lwc_writeLog (	LOG_CRIT,
+					_("Network error: select: %s"),
+					strerror (errno));
+			errno = save_errno;
+			return -1;
+		}
+		waiting_set = rfds;
+		still_waiting = ret;
+
+		s = net_accept_next (	&waiting_set, sock_max, &still_waiting,
+					allow_from, &sockname, &len);
+		if (s == -1) {
+			return -1;
+		}
+		if (s >= 0) {
+			if (names != NULL) {
+				build_client_names_from_addr (	&sockname, len,
+								names);
+			}
+			return s;
+		}
+	}
+	return -1;
+}
+
+
 /*
  * Manage all the network request on the provided socket.  For each request
  * a new thread is started.
- * The run function is called to interract with the client (its second
+ * The run function is called to interract with the agent (its second
  * parameter is the provided user_data parameter).
  *
  * Return:
@@ -269,21 +510,19 @@ net_utils_server_thread (void *data)
  *      lwc_writeLog())
  */
 int
-net_accept (int sock, int (*run)(schedwi_BIO *, void *), void *user_data)
+net_accept (	fd_set *sock_set, int sock_max, const char *allow_from,
+		int (*run)(schedwi_BIO *, void *), void *user_data)
 {
 	pthread_attr_t attr;
 	pthread_t thread;
-	int s, save_errno;
 	net_utils_data_ptr ptr;
-	struct sockaddr_in sockname;
-#if HAVE_SOCKLEN_T
+	int s, ret, save_errno;
+	fd_set rfds;
+	struct sockaddr_storage sockname;
 	socklen_t len;
-#else
-	int len;
-#endif
 
 #if HAVE_ASSERT_H
-	assert (run != NULL);
+	assert (run != NULL && sock_set != NULL);
 #endif
 
 	/* Initialize the thread attribute object */
@@ -303,37 +542,54 @@ net_accept (int sock, int (*run)(schedwi_BIO *, void *), void *user_data)
 	 */
 	while (1) {
 		pthread_testcancel ();
-		len = sizeof(struct sockaddr_in);
-		s = accept (sock, (struct sockaddr *)&sockname, &len);
-		if (s < 0) {
+		rfds = *sock_set;
+		ret = select (sock_max, &rfds, NULL, NULL, NULL);
+		if (ret < 0) {
 			if (errno == EINTR || errno == EAGAIN) {
-				/* Retry */
 				continue;
 			}
 			save_errno = errno;
 			lwc_writeLog (	LOG_CRIT,
-					_("Network error: accept: %s"),
+					_("Network error: select: %s"),
 					strerror (errno));
 			pthread_attr_destroy (&attr);
 			errno = save_errno;
 			return -1;
 		}
 
-		ptr = net_utils_data_new (s, &sockname, run, user_data);
-		if (ptr == NULL) {
-			closesocket (s);
-			lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-			continue;
-		}
-		if (pthread_create (	&thread, &attr,
-					net_utils_server_thread, ptr) != 0)
-		{
-			lwc_writeLog (	LOG_CRIT,
+		do {
+			pthread_testcancel ();
+			s = net_accept_next (	&rfds, sock_max,
+						&ret, allow_from,
+						&sockname, &len);
+			if (s == -1) {
+				save_errno = errno;
+				pthread_attr_destroy (&attr);
+				errno = save_errno;
+				return -1;
+			}
+			if (s >= 0) {
+				ptr = net_utils_data_new (s, &sockname, len,
+							run, user_data);
+				if (ptr == NULL) {
+					closesocket (s);
+					lwc_writeLog (	LOG_CRIT,
+						_("Memory allocation error"));
+					continue;
+				}
+				if (pthread_create (	&thread, &attr,
+							net_utils_server_thread,
+							ptr) != 0)
+				{
+					lwc_writeLog (	LOG_CRIT,
 _("Cannot create a new thread to manage the network connection: %s"),
-					strerror (errno));
-			net_utils_data_destroy (ptr); /* Also close `s' */
-			continue;
-		}
+						strerror (errno));
+					net_utils_data_destroy (ptr);
+					continue;
+				}
+			}
+
+		} while (s != -2);
 	}
 	/* Never reach this point */
 	pthread_attr_destroy (&attr);
@@ -368,9 +624,12 @@ net_init_server ()
 int
 net_init (	const char *schedwisrv_crt_file,
 		const char *schedwisrv_key_file,
+		const char *schedwisrv_ca_file,
+		const char *schedwisrv_crl_file,
 		char quick_random)
 {
 	return net_init_ssl (	schedwisrv_crt_file, schedwisrv_key_file,
+				schedwisrv_ca_file, schedwisrv_crl_file,
 				quick_random);
 }
 
@@ -413,7 +672,7 @@ net_client (	const char *port_number, const char *hostname, char use_ssl,
 		return NULL;
 	}
 
-	/* 
+	/*
 	 * No SSL, only socket
 	 */
 	if (use_ssl == 0) {
@@ -465,7 +724,7 @@ net_client (	const char *port_number, const char *hostname, char use_ssl,
  *       lwc_writeLog())
  */
 int
-net_read (schedwi_BIO *in, char **buff, unsigned int *len)
+net_read (schedwi_BIO *in, char **buff, size_t *len)
 {
 #if HAVE_ASSERT_H
 	assert (in != NULL && buff != NULL && len != NULL);
@@ -483,10 +742,12 @@ net_read (schedwi_BIO *in, char **buff, unsigned int *len)
 /*
  * Read the reply
  *
- * The format of the reply must be as follow:
- *    The first character: '0' means success, an other character means failure
- *    The next characters: An optional explanation message (which is returned
- *                         in result_msg)
+ * The format of the JSON reply looks like:
+ *     {
+ *         "success" : false,
+ *          "data" : "Database unavailable"
+ *     }
+ *
  * No reply (nothing to read) means failure.
  *
  * Return:
@@ -501,10 +762,9 @@ net_read (schedwi_BIO *in, char **buff, unsigned int *len)
 int
 net_read_result (schedwi_BIO *in, char **result_msg)
 {
-	char *buff, ret_code;
-	unsigned int len;
-	int nb_read;
-	int i;
+	char *buff;
+	size_t len;
+	int nb_read, ret;
 
 	if (result_msg != NULL) {
 		*result_msg = NULL;
@@ -522,32 +782,15 @@ net_read_result (schedwi_BIO *in, char **result_msg)
 	}
 
 	if (nb_read > 0) {
-		/* Return code sent by the remote process */
-		ret_code = buff[0];
-
-		if (result_msg != NULL) {
-			/* Shift the message 1 character to the left */
-			for (i = 1; i < nb_read; i++) {
-				buff[i - 1] = buff[i];
-			}
-			buff [nb_read - 1] = '\0';
-			*result_msg = buff;
-		}
-		else {
-			free (buff);
-		}
-
-		/* Return code '0' means success */
-		if (ret_code == '0') {
-			return 0;
-		}
+		ret = parse_json_result (buff, (ssize_t)nb_read, result_msg);
 	}
 	else {
-		if (buff != NULL) {
-			free (buff);
-		}
+		ret = 1;
 	}
-	return 1;
+	if (buff != NULL) {
+		free (buff);
+	}
+	return ret;
 }
 
 
@@ -579,22 +822,7 @@ net_write (schedwi_BIO *out, const char *buff, int len)
 
 
 /*
- * Write the End of Request tag
- *
- * Return:
- *   0 --> No error
- *  -1 --> Write error (a message is displayed using lwc_writeLog())
- */
-int
-net_write_eof (schedwi_BIO *out)
-{
-	return net_write (out, ENF_OF_REQUEST,
-				(int)schedwi_strlen (ENF_OF_REQUEST));  
-}
-
-
-/*
- * Close the handler 
+ * Close the handler
  */
 void
 net_close (schedwi_BIO *out)
