@@ -1178,7 +1178,8 @@ net_close_sock_set (fd_set *sock_set, int *sock_max)
 
 	if (sock_set != NULL) {
 		while (*sock_max > 0) {
-			if (FD_ISSET (--(*sock_max), sock_set) != 0) {
+			--(*sock_max);
+			if (FD_ISSET (*sock_max, sock_set) != 0) {
 				net_close_sock (*sock_max);
 			}
 		}
