@@ -39,23 +39,27 @@ extern "C" {
 
 extern int net_server_ssl OF((gnutls_session_t *session, int fd,
 		gnutls_certificate_credentials_t *trusted_cert_ptr,
-		char **names,
+		const char *const *names,
 		char *remote_crt, unsigned long int remote_crt_len));
+extern int net_init_gnutls OF((char quick_random));
 extern int net_init_server_ssl OF(());
 extern void net_destroy_ssl OF(());
 extern int net_init_ssl OF((	const char *schedwisrv_crt_file,
 				const char *schedwisrv_key_file,
+				const char *schedwisrv_ca_file,
+				const char *schedwisrv_crl_file,
 				char quick_random));
 extern int net_client_ssl OF((const char *port_number, const char *hostname,
 		gnutls_session_t *session, int *fd,
 		gnutls_certificate_credentials_t *trusted_cert_ptr,
 		char *remote_crt, unsigned int remote_crt_len));
 extern ssize_t net_read_ssl OF((gnutls_session_t session,
-				char **buff, unsigned int *len));
+				char **buff, size_t *len));
 extern int net_write_ssl OF((	gnutls_session_t session,
 				const char *buff, size_t len));
 extern int net_close_ssl OF((	gnutls_session_t session, int fd,
 			gnutls_certificate_credentials_t *trusted_cert_ptr));
+extern int check_string_request_names OF((char *s, const char *const *names));
 
 #ifdef __cplusplus
 }
