@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -17,7 +17,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* request.c -- Common functions to build a request to a client */
+/* request.c -- Common functions to build a request to a agent */
 
 #include <schedwi.h>
 
@@ -63,11 +63,11 @@ environment_callback (	void *obj, unsigned int pos,
 }
 
 /*
- * Error callback function for the sql_get_environment() function
+ * Error callback function for the sql_get_environment*() function
  */
 static void
 sql_get_environment_error_logger (	void *data, const char *msg,
-					unsigned int err_code)
+					int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
@@ -84,26 +84,26 @@ _("Database error while trying to retrieve a job environment variables"));
  *
  * Return:
  *   0 --> No error.  host_id_out, request_out and hierarchy_list_out are set
- *         by this function and must be freed by the caller by
- *             free (host_id_out)
+ *         by this function. request_out and hierarchy_list_out must be freed
+ *         by the caller by
  *             startjob_request_destroy(request_out)
  *             lwc_delLL(hierarchy_list_out, (void (*)(const void *))free)
  *  -1 --> Error.  An error message has been logged by lwc_writeLog().
  */
 int
 request_job_init (int workload_date, unsigned long long int job_id,
-		char **host_id_out,
+		unsigned long long int *host_id_out,
 		startjob_request_ptr *request_out,
 		lwc_LL **hierarchy_list_out)
 {
 	lwc_LL *hierarchy_list;
 	startjob_request_ptr request;
-	char workload_str[25], job_id_str[25], *host_id_str, *err_msg = NULL;
+	char workload_str[25], job_id_str[25], *err_msg = NULL;
+	row_item_t host_id;
 	unsigned int len, last_pos;
 
 #if HAVE_ASSERT_H
-	assert (   host_id_out != NULL
-		&& request_out != NULL && hierarchy_list_out != NULL);
+	assert (request_out != NULL && hierarchy_list_out != NULL);
 #endif
 
 	/* Create the startjob_request structure */
@@ -113,12 +113,8 @@ request_job_init (int workload_date, unsigned long long int job_id,
 		return -1;
 	}
 
-	/* Convert the job ID to a string */
-	len = copy_ulltostr (job_id, job_id_str);
-
 	/* Get the job hierarchy */
-	hierarchy_list = get_hierarchy_list (	workload_date, job_id_str,
-						&err_msg);
+	hierarchy_list = get_hierarchy_list (workload_date, job_id, &err_msg);
 	if (hierarchy_list == NULL) {
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -127,8 +123,8 @@ request_job_init (int workload_date, unsigned long long int job_id,
 		}
 		else {
 			lwc_writeLog (	LOG_CRIT,
-					_("Job %s: cannot retrieve details"),
-					job_id_str);
+					_("Job %lld: cannot retrieve details"),
+					job_id);
 		}
 		return -1;
 	}
@@ -136,6 +132,9 @@ request_job_init (int workload_date, unsigned long long int job_id,
 	/* Convert the workload date to a string */
 	copy_ulltostr (workload_date, workload_str);
 
+	/* Convert the job ID to a string */
+	len = copy_ulltostr (job_id, job_id_str);
+
 	/* Add the job ID to the startjob_request structure */
 	if (startjob_request_add_jobid (request, workload_str,
 					job_id_str, len) != 0)
@@ -148,7 +147,7 @@ request_job_init (int workload_date, unsigned long long int job_id,
 	/* Get the host ID associated with the job */
 	if  (get_job_parameter (workload_date, hierarchy_list,
 				"job_host_s", "host_id",
-				&host_id_str, NULL, &err_msg) != 0)
+				&host_id, &err_msg) != 0)
 	{
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
@@ -158,29 +157,29 @@ request_job_init (int workload_date, unsigned long long int job_id,
 		}
 		else {
 			lwc_writeLog (LOG_CRIT,
-		_("Job %s: error while retrieving the client host name"),
+		_("Job %s: error while retrieving the agent host name"),
 				job_id_str);
 		}
 		return -1;
 	}
-	if (host_id_str == NULL) {
+	if (host_id.type == RES_END) {
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		lwc_writeLog (LOG_CRIT,
-			_("Job %s: client host name missing in the database"),
+			_("Job %s: agent host name missing in the database"),
 				job_id_str);
 		return -1;
 	}
 
 	/* Get the host environment */
-	if (sql_get_environment_host (	workload_date, host_id_str,
-					environment_callback,
-					request,
-					&last_pos,
-					sql_get_environment_error_logger,
-					NULL) != 0)
+	if (sql_get_environment_host (workload_date,
+			(unsigned long long int)sql_row_item2ll (&host_id),
+			environment_callback,
+			request,
+			&last_pos,
+			sql_get_environment_error_logger,
+			NULL) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		return -1;
@@ -194,13 +193,12 @@ request_job_init (int workload_date, unsigned long long int job_id,
 					sql_get_environment_error_logger,
 					NULL) != 0)
 	{
-		free (host_id_str);
 		lwc_delLL (hierarchy_list, (void (*)(const void *))free);
 		startjob_request_destroy (request);
 		return -1;
 	}
 
-	*host_id_out = host_id_str;
+	*host_id_out = (unsigned long long int)sql_row_item2ll (&host_id);
 	*request_out = request;
 	*hierarchy_list_out = hierarchy_list;
 	return 0;
@@ -221,18 +219,17 @@ request_job_init (int workload_date, unsigned long long int job_id,
 int
 request_testfile_init (	int workload_date,
 			unsigned long long int job_id,
-			const char *host_id_str,
+			unsigned long long int host_id,
 			const char *filename,
 			startjob_request_ptr *request_out)
 {
 	lwc_LL *hierarchy_list;
 	startjob_request_ptr request;
-	char job_id_str[25], *err_msg = NULL;
+	char *err_msg = NULL;
 	unsigned int last_pos;
 
 #if HAVE_ASSERT_H
-	assert (   host_id_str != NULL && filename != NULL
-		&& request_out != NULL);
+	assert (filename != NULL && request_out != NULL);
 #endif
 
 	/* Create the startjob_request structure */
@@ -251,12 +248,8 @@ request_testfile_init (	int workload_date,
 		return -1;
 	}
 
-	/* Convert the job ID to a string */
-	copy_ulltostr (job_id, job_id_str);
-
 	/* Get the job hierarchy */
-	hierarchy_list = get_hierarchy_list (	workload_date, job_id_str,
-						&err_msg);
+	hierarchy_list = get_hierarchy_list (workload_date, job_id, &err_msg);
 	if (hierarchy_list == NULL) {
 		startjob_request_destroy (request);
 		if (err_msg != NULL) {
@@ -265,15 +258,15 @@ request_testfile_init (	int workload_date,
 		}
 		else {
 			lwc_writeLog (	LOG_CRIT,
-					_("Job %s: cannot retrieve details"),
-					job_id_str);
+					_("Job %lld: cannot retrieve details"),
+					job_id);
 		}
 		return -1;
 	}
 
 	/* Get the host environment */
 	if (sql_get_environment_host (	workload_date,
-					host_id_str, environment_callback,
+					host_id, environment_callback,
 					request,
 					&last_pos,
 					sql_get_environment_error_logger,
