@@ -56,6 +56,10 @@ typedef int ssize_t;
 typedef long int time_t;
 #endif
 
+#ifndef HAVE_SOCKLEN_T
+typedef int socklen_t;
+#endif
+
 #endif /* _SCHEDWI_H */
 
 /*------------------------======= End Of File =======------------------------*/
