@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007-2010 Herve Quatremain
+   Copyright (C) 2007-2013 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -145,6 +145,11 @@ set_schedwi_jobtree_node (	schedwi_jobtree_node_ptr ptr,
     ptr->status = (current_status == 0)
                 ? JOB_STATUS_STATE_WAITING
                 : job_status_state_int2status (current_status);
+    if (current_status == 0) {
+        /* It's a new job (waiting). Add an entry in the job_status table */
+        job_status_set (ptr->id, ptr->path, workload_date,
+                        JOB_STATUS_STATE_WAITING, 0, 0, 0, NULL);
+    }
     ptr->start_time = (ptr->status == JOB_STATUS_STATE_RUNNING)
                                                ? (schedwi_time) start_time : 0;
     ptr->run_time   = schedwi_time_convert (job_date, job_time);
@@ -1282,7 +1287,8 @@ schedwi_jobtree_node_set_status (	schedwi_date workload_date,
 	}
 
 	/* Update the status in the database */
-	ret = job_status_set (ptr->id, ptr->path, workload_date, status,
+	ret = job_status_set (ptr->id, ptr->path,
+			schedwi_date_to_int (workload_date), status,
 			current_time,
 			(duration >= 0)	? duration
 					: current_time - ptr->start_time,
