@@ -51,9 +51,11 @@ struct _schedwi_jobtree_node {
 
 	unsigned long long int id;
 	char *path;               /* Full job/jobset name (with its path) */
-	schedwi_time run_time;    /* Start date and time of this job/jobset */
+	schedwi_time run_time;    /* Start time of this job/jobset (EPOCH) */
 	schedwi_time start_time;  /* Real start time */
 
+    schedwi_date start_day;   /* Start day */
+
 	/* Max number of seconds for this job/jobset to be started */
 	long int start_limit;
 
