@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2011 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -74,6 +74,7 @@ extern int errno;
 #include <lwc_log.h>
 #include <net_utils.h>
 #include <net_utils_sock.h>
+#include <net_utils_ssl.h>
 #include <net_parse.h>
 #include <schedwi_time.h>
 #include <load_workload_class.h>
@@ -82,6 +83,7 @@ extern int errno;
 #include <commands_thread.h>
 #include <module.h>
 #include <lib_functions.h>
+#include <cert_ca.h>
 
 
 static const char *pid_file = NULL;
@@ -89,6 +91,12 @@ static const char *configuration_file = NULL;
 static pthread_t net_server_thread;
 static load_workload_class_ptr workload_list = NULL;
 
+struct socket_set {
+	fd_set sock_set;
+	int sock_max;
+	const char *allow_from;
+};
+
 
 
 /*
@@ -137,7 +145,7 @@ NULL
 #endif
 
 	fprintf (stderr, _("Usage: %s [OPTION]...\n"), prog_name);
-						
+
 	for (i = 0; msg[i] != NULL; i++) {
 		if (msg[i][0] == '~') {
 			fputs (	_("The default configuration file is "),
@@ -171,13 +179,11 @@ sighup_action ()
 	 * Before anything else be sure that the configuration file
 	 * can be read
 	 */
-#if HAVE_ACCESS && defined R_OK
 	if (access (configuration_file, R_OK) != 0) {
 		lwc_writeLog (	LOG_ERR, _("%s: %s"),
 				configuration_file, strerror (errno));
 		return;
 	}
-#endif
 
 	close_pid_file (pid_file);
 	conf_destroy_srv ();
@@ -225,11 +231,11 @@ sighup_action ()
 	assert (ret == 0);
 #endif
 
-        lwc_newLog (    PACKAGE_NAME,
-                        (facility != -1) ? 1 : 0,
-                        (facility != -1) ? (int)facility : 0,
-                        log_file,
-                        date_format);
+	lwc_newLog (    PACKAGE_NAME,
+			(facility != -1) ? 1 : 0,
+			(facility != -1) ? (int)facility : 0,
+			log_file,
+			date_format);
 	lwc_writeLog (LOG_INFO, _("Restarting"));
 
 	/*
@@ -302,7 +308,7 @@ sigalrm_action()
 			previous = now;
 		}
 		else {
-			/* 
+			/*
 			 * Failed to load the workload.
 			 * Try again in a minute
 			 */
@@ -321,9 +327,9 @@ sigalrm_action()
 static void *
 net_server_thread_func (void *data)
 {
-	int *sock = (int *)data;
+	struct socket_set *s_set = (struct socket_set *)data;
 
-	if (sock == NULL) {
+	if (s_set == NULL) {
 		pthread_exit (NULL);
 	}
 
@@ -334,7 +340,8 @@ net_server_thread_func (void *data)
 	pthread_cleanup_push (thread_clean, NULL);
 
 	/* Manage network connections */
-	net_accept (*sock, net_parse, workload_list);
+	net_accept (	&(s_set->sock_set), s_set->sock_max, s_set->allow_from,
+			net_parse, workload_list);
 
 	pthread_cleanup_pop (1);
 	pthread_exit (NULL);
@@ -342,10 +349,10 @@ net_server_thread_func (void *data)
 
 
 /*
- * Create the thread to manage the client connections
+ * Create the thread to manage the agent connections
  */
 static int
-start_net_server (int *sock)
+start_net_server (struct socket_set *s_set)
 {
 	pthread_attr_t attr;
 
@@ -354,7 +361,7 @@ start_net_server (int *sock)
 	pthread_attr_setdetachstate (&attr, PTHREAD_CREATE_JOINABLE);
 
 	if (pthread_create (	&net_server_thread, &attr,
-				net_server_thread_func, sock) != 0)
+				net_server_thread_func, s_set) != 0)
 	{
 		lwc_writeLog (	LOG_CRIT,
 	_("Cannot create a new thread to manage network connections: %s"),
@@ -375,19 +382,20 @@ start_net_server (int *sock)
 int
 main (int argc, char **argv)
 {
-	int sock, ret, sig;
+	int ret, sig;
 	const char *group_id, *user_id;
 	const char *log_file, *date_format;
-	const char *iface, *server_port;
-	const char *server_key, *server_crt;
+	const char *iface, *server_port, *address_family;
+	const char *server_key, *server_crt, *ca_crt, *ca_key, *ca_crl;
 	char quick_random;
 	const char *prog_name;
 	long int facility;
 	char test, foreground;
-	MYSQL *sql_handler;
+	dbi_conn sql_handler;
 	char *err_msg;
 	schedwi_date now;
 	sigset_t waitset;
+	struct socket_set s_set;
 
 #if HAVE_GETOPT_LONG
 	int option_index;
@@ -484,19 +492,55 @@ main (int argc, char **argv)
 		return 1;
 	}
 
-	/*
-	 * Initialize the network (before detaching the program from its
-	 * terminal as a password may be asked for the SSL private key)
-	 */
-	ret = conf_get_param_string ("SSLServerCertificateFile", &server_crt);
-	ret += conf_get_param_string (	"SSLServerCertificateKeyFile",
-					&server_key);
+	ret =  conf_get_param_string ("SSLCACertificateFile", &ca_crt);
+	ret += conf_get_param_string ("SSLCACertificateKeyFile", &ca_key);
+	ret += conf_get_param_string ("SSLCACRLFile", &ca_crl);
+	ret += conf_get_param_string ("SSLCertificateFile", &server_crt);
+	ret += conf_get_param_string ("SSLCertificateKeyFile", &server_key);
 	ret += conf_get_param_bool ("SSLQuickRandom", &quick_random);
+	ret += conf_get_param_string ("USER", &user_id);
+	ret += conf_get_param_string ("GROUP", &group_id);
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
 
-	if (net_init (server_crt, server_key, quick_random) != 0) {
+	/*
+	 * Initialize the GnuTLS layer
+	 */
+	if (net_init_gnutls (quick_random) != 0) {
+		conf_destroy_srv ();
+		module_exit ();
+		return 1;
+	}
+
+	/*
+	 * Create the keys and certificates if needed
+	 */
+	if (	   test == 0
+		&& cert_check_and_create (	ca_crt, ca_key,
+						server_crt, server_key,
+						user_id, group_id,
+						quick_random) != 0)
+	{
+		net_destroy ();
+		conf_destroy_srv ();
+		module_exit ();
+		return 1;
+	}
+
+	/*
+	 * Initialize the network (before detaching the program from its
+	 * terminal as a password may be asked for the SSL private key)
+	 */
+	if ((	   test == 0
+		|| (	   access (ca_crt, F_OK) == 0
+			&& access (server_crt, F_OK) == 0
+			&& access (server_key, F_OK) == 0
+		   )
+	    ) && net_init (	server_crt, server_key, ca_crt,
+				ca_crl, quick_random) != 0)
+	{
+		net_destroy ();
 		conf_destroy_srv ();
 		module_exit ();
 		return 1;
@@ -505,11 +549,6 @@ main (int argc, char **argv)
 	/*
 	 * Change the group id and the user id of the process
 	 */
-	ret = conf_get_param_string ("GROUP", &group_id);
-	ret += conf_get_param_string ("USER", &user_id);
-#if HAVE_ASSERT_H
-	assert (ret == 0);
-#endif
 	ret = change_group (group_id);
 	if (ret != 0) {
 		if (ret == -2) {
@@ -552,7 +591,7 @@ main (int argc, char **argv)
 
 		/* Just try a connection to the database */
 		err_msg = NULL;
-		sql_handler = begin_mysql (&err_msg);
+		sql_handler = begin_sql (&err_msg);
 		if (sql_handler == NULL) {
 			if (err_msg != NULL) {
 				fputs (err_msg, stderr);
@@ -567,7 +606,7 @@ main (int argc, char **argv)
 			module_exit ();
 			return 1;
 		}
-		end_mysql ();
+		end_sql ();
 
 		conf_destroy_srv ();
 		module_exit ();
@@ -605,11 +644,11 @@ main (int argc, char **argv)
 	assert (ret == 0);
 #endif
 
-        lwc_newLog (    PACKAGE_NAME,
-                        (facility != -1) ? 1 : 0,
-                        (facility != -1) ? (int)facility : 0,
-                        log_file,
-                        date_format);
+	lwc_newLog (    PACKAGE_NAME,
+			(facility != -1) ? 1 : 0,
+			(facility != -1) ? (int)facility : 0,
+			log_file,
+			date_format);
 	lwc_writeLog (LOG_INFO, _("Starting"));
 
 
@@ -648,7 +687,7 @@ main (int argc, char **argv)
 	 * Database connection
 	 */
 	err_msg = NULL;
-	sql_handler = begin_mysql (&err_msg);
+	sql_handler = begin_sql (&err_msg);
 	if (sql_handler == NULL) {
 		if (err_msg != NULL) {
 			lwc_writeLog (	LOG_CRIT,
@@ -672,18 +711,22 @@ main (int argc, char **argv)
 	/*
 	 * SSL setup
 	 */
-	lwc_writeLog (LOG_INFO,
+	if (quick_random == 0) {
+		lwc_writeLog (LOG_INFO,
 		_("Initializing the SSL layer (may take several minutes)..."));
+	}
 	if (net_init_server () != 0) {
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		net_destroy ();
 		module_exit ();
 		return 1;
 	}
-	lwc_writeLog (LOG_INFO, _("Done"));
+	if (quick_random == 0) {
+		lwc_writeLog (LOG_INFO, _("Done"));
+	}
 
 
 	/*
@@ -691,15 +734,20 @@ main (int argc, char **argv)
 	 */
 	ret = conf_get_param_string ("IFACE_LISTEN", &iface);
 	ret += conf_get_param_string ("SERVER_PORT", &server_port);
+	ret += conf_get_param_string ("ADDRESS_FAMILY", &address_family);
+	ret += conf_get_param_string ("ALLOW_FROM", &(s_set.allow_from));
 #if HAVE_ASSERT_H
 	assert (ret == 0);
 #endif
 
-	sock = net_server (server_port, iface);
-	if (sock < 0) {
+	FD_ZERO (&(s_set.sock_set));
+	s_set.sock_max = -1;
+	if (net_server (server_port, iface, address_family,
+			&(s_set.sock_set), &(s_set.sock_max)) != 0)
+	{
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
+		end_sql ();
 		conf_destroy_srv ();
 		net_destroy ();
 		module_exit ();
@@ -721,8 +769,8 @@ main (int argc, char **argv)
 	if (workload_list == NULL) {
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
-		net_close_sock (sock);
+		end_sql ();
+		net_close_sock_set (&(s_set.sock_set), &(s_set.sock_max));
 		conf_destroy_srv ();
 		net_destroy ();
 		module_exit ();
@@ -732,8 +780,8 @@ main (int argc, char **argv)
 	if (load_workload_class_from_status_table (workload_list) != 0) {
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
-		net_close_sock (sock);
+		end_sql ();
+		net_close_sock_set (&(s_set.sock_set), &(s_set.sock_max));
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
@@ -751,8 +799,8 @@ main (int argc, char **argv)
 	{
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
-		net_close_sock (sock);
+		end_sql ();
+		net_close_sock_set (&(s_set.sock_set), &(s_set.sock_max));
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
@@ -763,11 +811,11 @@ main (int argc, char **argv)
 	/*
 	 * Start the server that will listen to network requests
 	 */
-	if (start_net_server (&sock) != 0) {
+	if (start_net_server (&s_set) != 0) {
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
-		net_close_sock (sock);
+		end_sql ();
+		net_close_sock_set (&(s_set.sock_set), &(s_set.sock_max));
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
@@ -782,8 +830,8 @@ main (int argc, char **argv)
 		pthread_join (net_server_thread, NULL);
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
-		net_close_sock (sock);
+		end_sql ();
+		net_close_sock_set (&(s_set.sock_set), &(s_set.sock_max));
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
 		net_destroy ();
@@ -806,8 +854,8 @@ main (int argc, char **argv)
 		pthread_join (net_server_thread, NULL);
 		lwc_closeLog ();
 		close_pid_file (pid_file);
-		end_mysql ();
-		net_close_sock (sock);
+		end_sql ();
+		net_close_sock_set (&(s_set.sock_set), &(s_set.sock_max));
 		commands_thread_destroy ();
 		load_workload_class_destroy (workload_list);
 		conf_destroy_srv ();
@@ -835,8 +883,9 @@ main (int argc, char **argv)
 			pthread_join (net_server_thread, NULL);
 			lwc_closeLog ();
 			close_pid_file (pid_file);
-			end_mysql ();
-			net_close_sock (sock);
+			end_sql ();
+			net_close_sock_set (	&(s_set.sock_set),
+						&(s_set.sock_max));
 			commands_thread_destroy ();
 			load_workload_class_destroy (workload_list);
 			conf_destroy_srv ();
@@ -858,11 +907,12 @@ main (int argc, char **argv)
 				break;
 
 			case SIGTERM:
-				/* Stop listening for client requests */
+				/* Stop listening for agent requests */
 				pthread_cancel (net_server_thread);
 				pthread_kill (net_server_thread, SIGUSR2);
 				pthread_join (net_server_thread, NULL);
-				net_close_sock (sock);
+				net_close_sock_set (	&(s_set.sock_set),
+							&(s_set.sock_max));
 
 				/* Stop user commands */
 				commands_thread_destroy ();
@@ -871,7 +921,7 @@ main (int argc, char **argv)
 				load_workload_class_destroy (workload_list);
 
 				/* Disconnect from the database */
-				end_mysql ();
+				end_sql ();
 
 				/* Destroy the configuration file object */
 				conf_destroy_srv ();
