@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -30,26 +30,26 @@
 extern "C" {
 #endif
 
-extern unsigned int sql_command_get_commands OF((
-		int (*callback)(void *, const char *, const char *,
+extern int sql_command_get_commands OF((
+		int (*callback)(void *, unsigned int, unsigned long long int,
 				const char *, int, int),
 		void *user_data,
-		void (*error_func)(void *, const char *, unsigned int),
+		void (*error_func)(void *, const char *, int),
 		void *user_data_error_func));
-extern unsigned int sql_command_add OF((int workload_date, const char *job_id,
-					const char *user, int command,
-					int parameter, char **err_msg));
-extern unsigned int sql_command_done OF((
+extern int sql_command_add OF((int workload_date, const char *job_id,
+				const char *user, int command,
+				int parameter, char **err_msg));
+extern int sql_command_done OF((
 			int workload_date, const char *job_id,
 			int command, int parameter,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_command_failed OF((
+extern int sql_command_failed OF((
 			int workload_date, const char *job_id,
 			int command, int parameter,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_command_purge OF((char **err_msg));
+extern int sql_command_purge OF((char **err_msg));
 
 #ifdef __cplusplus
 }
