@@ -746,11 +746,16 @@ sql_free_row (row_item_t *row)
 long long int
 sql_row_item2ll (const row_item_t *item)
 {
-	if (item != NULL && (item->type == RES_INT || item->type == RES_UINT))
-	{
-		return (long long int)((item->type == RES_INT)
+	if (item != NULL) {
+		if (item->type == RES_INT || item->type == RES_UINT) {
+			return (long long int)((item->type == RES_INT)
 							? item->value_int
 							: item->value_uint);
+		}
+		if (item->type == RES_STRING) {
+			return (long long int)strtoull(item->value_string,
+							NULL, 0);
+		}
 	}
 	return 0;
 }
