@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -37,7 +37,7 @@
 #include <sql_constraint_file.h>
 
 
-#define SQL_GET_CONSTRAINT_FILE "SELECT job_id,host_id,filename,exist FROM constraint_file_s WHERE workload_date=%d AND job_id=\"%s\""
+#define SQL_GET_CONSTRAINT_FILE "SELECT job_id,host_id,filename,exist FROM constraint_file_s WHERE workload_date=%d AND job_id=%ld"
 
 
 /*
@@ -47,29 +47,30 @@
  *     0 --> No error.  rows contains the retrieved files and must be freed
  *           by the caller by
  *              lwc_delLL (rows, (void (*)(const void *)) sql_free_row);
- *     1 --> Memory allocation error (if error_func() is not NULL, it is called
+ *    -1 --> Memory allocation error (if error_func() is not NULL, it is called
  *           with user_data_error_func as its first parameter and the error
  *           message as the second parameter)
- * other --> SQL error (if error_func() is not NULL, it is called with
+ *    -2 --> SQL error (if error_func() is not NULL, it is called with
  *           user_data_error_func as its first parameter and the error message
  *           as the second parameter)
  */
-unsigned int
-sql_constraint_file_get (int workload_date, lwc_LL **rows, const char *job_id,
-			void (*error_func)(void *, const char *, unsigned int),
+int
+sql_constraint_file_get (int workload_date, lwc_LL **rows,
+			unsigned long long int job_id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func)
 {
 	char *err_msg = NULL;
-	unsigned int ret;
+	int ret;
 
 #if HAVE_ASSERT_H
-	assert (rows != NULL && job_id != NULL);
+	assert (rows != NULL);
 #endif
 
-	ret = sql_select (      NULL, NULL, &err_msg, NULL, rows, NULL,
+	ret = sql_select (      NULL, NULL, &err_msg, NULL, rows,
 				SQL_GET_CONSTRAINT_FILE,
 				SQL_INT, (long int)workload_date,
-				SQL_STRING, job_id,
+				SQL_INT, (long int)job_id,
 				SQL_END);
 
 	if (ret != 0) {
