@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -31,10 +31,11 @@
 extern "C" {
 #endif
 
-extern unsigned int sql_constraint_file_get OF((
+extern int sql_constraint_file_get OF((
 			int workload_date,
-			lwc_LL **rows, const char *job_id,
-			void (*error_func)(void *, const char *, unsigned int),
+			lwc_LL **rows,
+			unsigned long long int job_id,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
 
 #ifdef __cplusplus
