@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -34,19 +34,19 @@
 extern "C" {
 #endif
 
-extern unsigned int sql_get_environment OF((int workload_date,
+extern int sql_get_environment OF((int workload_date,
 	lwc_LL *lst, unsigned int next_pos,
 	int (*callback)(void *, unsigned int, const char *, const char *),
 	void *user_data,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func));
 
-extern unsigned int sql_get_environment_host OF((int workload_date,
-	const char *host_id, 
+extern int sql_get_environment_host OF((int workload_date,
+	unsigned long long int host_id,
 	int (*callback)(void *, unsigned int, const char *, const char *),
 	void *user_data,
 	unsigned int *next_pos,
-	void (*error_func)(void *, const char *, unsigned int),
+	void (*error_func)(void *, const char *, int),
 	void *user_data_error_func));
 
 
