@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -30,11 +30,11 @@
 extern "C" {
 #endif
 
-extern unsigned int sql_link_get_list OF((
+extern int sql_link_get_list OF((
 			lwc_LL **rows,
 			int workload_date,
 			const char *id_list,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
 
 #ifdef __cplusplus
