@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -31,35 +31,40 @@
 extern "C" {
 #endif
 
-extern unsigned int sql_status_workload_list OF((lwc_LL **rows,
-			void (*error_func)(void *, const char *, unsigned int),
+extern int sql_status_workload_list OF((lwc_LL **rows,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_status_get OF((int workload_date,
-			const char *job_id,
+extern int sql_status_get OF((int workload_date,
+			unsigned long long int job_id,
 			int *status,
 			long int *start_time,
 			int *retry_num,
 			char **message,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_status_set OF((int workload_date,
+extern int sql_status_set OF((int workload_date,
 			const char *job_id,
 			int status,
 			unsigned int start_time,
 			int retry_num,
 			const char *error_message,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_status_delete OF((int workload_date,
-			void (*error_func)(void *, const char *, unsigned int),
+extern int sql_status_adjust_start_time OF((int workload_date,
+			unsigned long long int job_id,
+			unsigned int start_time,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_status_get_old OF((int workload_date,
-			int (*callback)(void *, const char *, const char *),
+extern int sql_status_delete OF((int workload_date,
+			void (*error_func)(void *, const char *, int),
+			void *user_data_error_func));
+extern int sql_status_get_old OF((int workload_date,
+			int (*callback)(void *, const char *, int),
 			void *user_data,
-			void (*error_func)(void *, const char *, unsigned int),
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
-extern unsigned int sql_status_copy_to_workload OF((int workload_date,
-			void (*error_func)(void *, const char *, unsigned int),
+extern int sql_status_copy_to_workload OF((int workload_date,
+			void (*error_func)(void *, const char *, int),
 			void *user_data_error_func));
 
 #ifdef __cplusplus
