@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007-2010 Herve Quatremain
+   Copyright (C) 2007-2012 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -51,6 +51,7 @@ extern int errno;
 #include <sql_hierarchy.h>
 #include <sql_hosts.h>
 #include <lib_functions.h>
+#include <memsend.h>
 #include <stopjob.h>
 
 /*
@@ -97,7 +98,7 @@ stopjob (	int workload_date, unsigned long long int job_id,
 	char *err_msg = NULL, *req;
 	lwc_LL *hierarchy_list;
 	row_item_t *host_details;
-	char *s;
+	memsend_t *membuff;
 
 
 	/* Get the job hierarchy */
@@ -181,20 +182,33 @@ stopjob (	int workload_date, unsigned long long int job_id,
 	}
 
 	/* Send the request */
-	s = "[ \"killjob\", { \"jobs\" : [ \"";
-	if (	   net_write (b, s, schedwi_strlen (s)) != 0
-		|| net_write (b, req, (int)len) != 0)
-	{
+	membuff = memsend_new ();
+	if (membuff == NULL) {
 		net_close (b);
 		free (req);
+		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
+		return -1;
+	}
+
+	if (memsend_append (membuff, "[ \"killjob\", { \"jobs\" : [ \"") != 0
+		|| memsend_append_len (membuff, req, len) != 0
+		|| memsend_append (membuff, "\" ] } ]") != 0)
+	{
+		memsend_destroy (membuff);
+		net_close (b);
+		free (req);
+		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
 		return -1;
 	}
 	free (req);
-	s = "\" ] } ]";
-	if (net_write (b, s, schedwi_strlen (s)) != 0) {
+	if (net_write (b, memsend_getstring (membuff),
+			memsend_getlength(membuff)) != 0)
+	{
+		memsend_destroy (membuff);
 		net_close (b);
 		return -1;
 	}
+	memsend_destroy (membuff);
 
 	/* Read the result */
 	ret_read = net_read_result (b, out_message);
