@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -17,7 +17,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-/* testfile.c -- Test the presence of a file on a client */
+/* testfile.c -- Test the presence of a file on a agent */
 
 #include <schedwi.h>
 
@@ -47,7 +47,6 @@ extern int errno;
 #include <utils.h>
 #include <lwc_log.h>
 #include <net_utils.h>
-#include <net_module_name.h>
 #include <sql_common.h>
 #include <sql_hosts.h>
 #include <request.h>
@@ -59,14 +58,14 @@ extern int errno;
  */
 static void
 sql_host_get_main_row_error_logger (	void *data, const char *msg,
-					unsigned int err_code)
+					int err_code)
 {
 	if (msg != NULL) {
 		lwc_writeLog (LOG_ERR, msg);
 	}
 	else {
 		lwc_writeLog (LOG_ERR,
-		_("Database error while retrieving the client host details"));
+		_("Database error while retrieving the agent host details"));
 	}
 }
 
@@ -76,12 +75,12 @@ sql_host_get_main_row_error_logger (	void *data, const char *msg,
  *
  * Return:
  *   0 --> No error.  The file exists on the remote host.  If not
- *         NULL, out_message contains a reply message sent by the client.  It
- *         may be NULL if the client didn't send back a message.  out_message
+ *         NULL, out_message contains a reply message sent by the agent.  It
+ *         may be NULL if the agent didn't send back a message.  out_message
  *         must be freed by the caller by free()
- *   1 --> The file does not exists on the remote client (or error on the
- *         client side).  If not NULL, out_message contains a reply message
- *         sent by the client.  It may be NULL if the client didn't send back
+ *   1 --> The file does not exists on the remote agent (or error on the
+ *         agent side).  If not NULL, out_message contains a reply message
+ *         sent by the agent.  It may be NULL if the agent didn't send back
  *         a message.  out_message must be freed by the caller by free()
  *  -1 --> Error (a message is logged using lwc_writeLog()).  If not NULL,
  *         out_message contains an error message.  It may be NULL if there
@@ -95,27 +94,18 @@ testfile (	int workload_date,
 		const char *filename,
 		char **out_message)
 {
-	char host_id_str[25];
 	startjob_request_ptr request;
 	char *err_msg = NULL;
-	char **host_details;
-	unsigned long int *host_details_len;
-	char *module;
-	int module_len, ret_read;
+	row_item_t *host_details;
+	int ret_read, ret;
 	schedwi_BIO *b;
-	unsigned int len, ret;
-	const char *req;
-
 
 #if HAVE_ASSERT_H
 	assert (filename != NULL);
 #endif
 
-	/* Convert the host ID to a string */
-	copy_ulltostr (host_id, host_id_str);
-
 	/* Build the request */
-	if (request_testfile_init (	workload_date, job_id, host_id_str,
+	if (request_testfile_init (	workload_date, job_id, host_id,
 					filename, &request) != 0)
 	{
 		if (out_message != NULL) {
@@ -131,8 +121,7 @@ testfile (	int workload_date,
 	}
 
 	/* Get the host details */
-	ret = sql_host_get_main_row (	host_id_str,
-					&host_details, &host_details_len,
+	ret = sql_host_get_main_row (	host_id, &host_details,
 					sql_host_get_main_row_error_logger,
 					NULL);
 	if (ret != 0) {
@@ -149,36 +138,14 @@ testfile (	int workload_date,
 		return -1;
 	}
 
-	/* Compose the module name */
-	module = NULL;
-	module_len = build_module_name ("testfile-0.1", &module);
-	if (module_len < 0) {
-		sql_free_row (host_details);
-		free (host_details_len);
-		startjob_request_destroy (request);
-		lwc_writeLog (LOG_CRIT, _("Memory allocation error"));
-		if (out_message != NULL) {
-			err_msg =
-				_("Server error: memory allocation error");
-			*out_message = (char *)malloc (
-						schedwi_strlen (err_msg) + 1);
-			if (*out_message != NULL) {
-				strcpy (*out_message, err_msg);
-			}
-		}
-		return -1;
-	}
-
-	/* Establish the connection to the client */
-	b = net_client (host_details[2],  /* TCP port */
-			host_details[1],  /* Client host name */
-			(host_details[3][0] == '0') ? 0: 1, /* SSL */
-			host_details[4],  /* Certificate */
-			(unsigned int)(host_details_len[4]));
-	free (host_details_len);
+	/* Establish the connection to the agent */
+	b = net_client (host_details[2].value_string,  /* TCP port */
+			host_details[1].value_string,  /* Agent host name */
+			(char)sql_row_item2ll (&(host_details[3])),
+			host_details[4].value_string,  /* Certificate */
+			(unsigned int)(host_details[4].len));
 	sql_free_row (host_details);
 	if (b == NULL) {
-		free (module);
 		startjob_request_destroy (request);
 		if (out_message != NULL) {
 			err_msg =
@@ -192,16 +159,9 @@ _("Server error: cannot establish the connection with the remote agent");
 		return -1;
 	}
 
-	/* Get the request string */
-	len = get_request_string (request, &req);
-
 	/* Send the request */
-	if (	   net_write (b, module, (int)module_len) != 0
-		|| net_write (b, req, (int)len) != 0
-		|| net_write_eof (b) != 0)
-	{
+	if (net_write_request (request, b, "testfile") != 0) {
 		net_close (b);
-		free (module);
 		startjob_request_destroy (request);
 		if (out_message != NULL) {
 			err_msg =
@@ -214,7 +174,6 @@ _("Server error: cannot establish the connection with the remote agent");
 		}
 		return -1;
 	}
-	free (module);
 	startjob_request_destroy (request);
 
 	/* Read the result */
