@@ -1,5 +1,5 @@
 /* Schedwi
-   Copyright (C) 2007 Herve Quatremain
+   Copyright (C) 2007-2010 Herve Quatremain
 
    This file is part of Schedwi.
 
@@ -169,7 +169,7 @@ void
 version (const char *prog_name)
 {
 	printf ("%s (Schedwi) %s\n", prog_name, PACKAGE_VERSION);
-	puts("Copyright (C) 2007 Herve Quatremain");
+	puts("Copyright (C) 2007-2010 Herve Quatremain");
 	puts("Schedwi is free software, covered by the GNU");
 	puts("General Public License version 3 or later.");
 	puts("You may redistribute copies of Schedwi under");
@@ -231,7 +231,7 @@ my_unlink (const char *file_name)
  *       - At the start of the agent, when it goes into background
  *       - Every time a job is started
  *
- *    In the Schedwi server (schedwisrc) it is just called once at the
+ *    In the Schedwi server (schedwisrv) it is just called once at the
  *    beginning, when the program goes into background.
  *
  *    Clear the environment may cause issues.  In particular, some variables
@@ -259,11 +259,11 @@ sanitise (int fd)
 	}
 
 	/*
-         * Redirect 0 (stdin), 1 (stdout) and 2 (stderr) to /dev/null.  This
-         * way, if sub-proccesses/modules try to inadvertently read/write to
-         * this file descriptors it will not mess other ones (pid file,
-         * socket, database connection, ...)
-         */
+	 * Redirect 0 (stdin), 1 (stdout) and 2 (stderr) to /dev/null.  This
+	 * way, if sub-proccesses/modules try to inadvertently read/write to
+	 * this file descriptors it will not mess other ones (pid file,
+	 * socket, database connection, ...)
+	 */
 	i = open ("/dev/null", O_RDWR); /* i should be 0 */
 	dup2 (i, 1);
 	dup2 (i, 2);
@@ -295,6 +295,7 @@ sanitise (int fd)
 void
 clean_process ()
 {
+	int unused;
 #if HAVE_CHDIR
 	char *tmp;
 #endif
@@ -312,9 +313,9 @@ clean_process ()
 #endif /* HAVE_GETENV */
 	if (tmp == NULL) {
 #ifdef P_tmpdir
-		chdir (P_tmpdir);
+		unused = chdir (P_tmpdir);
 #else
-		chdir ("/tmp");
+		unused = chdir ("/tmp");
 #endif
 	}
 #endif /* HAVE_CHDIR */
@@ -388,6 +389,7 @@ write_pid_file (const char *file_name)
 {
 	char pid_str[26];
 	size_t l;
+	ssize_t unused;
 
 	if (file_name == NULL) {
 		return 0;
@@ -416,7 +418,7 @@ write_pid_file (const char *file_name)
 	/* Write the PID to the file */
 	l = snprintf (pid_str, 26, "%ld\n", (long int)getpid ());
 	pid_str [26 - 1] = '\0';
-	write (fd_pid_file, pid_str, schedwi_strlen (pid_str));
+	unused = write (fd_pid_file, pid_str, schedwi_strlen (pid_str));
 	return 0;
 }
 
@@ -617,4 +619,65 @@ change_group (const char *name)
 #endif
 }
 
+
+/*
+ * Escape the provided string.  In this string, `"' are added
+ * around the argument (if add_quotes equal 1) and `"' and `\' in the
+ * argument are escaped by \.
+ *
+ * Return:
+ *    The argument string (to be freed by the caller) or
+ *    NULL on malloc error
+ */
+char *
+escape_string (const char *str, unsigned int str_len, char add_quotes)
+{
+	int i, j;
+	char *s;
+
+#ifdef HAVE_ASSERT_H
+	assert (str != NULL);
+#endif
+
+	/* Count the number of `"' */
+	i = 0;
+	s = strchr (str, '"');
+	while (s != NULL) {
+		i++;
+		s++;
+		s = strchr (s, '"');
+	}
+
+	/* Count the number of `\' */
+	s = strchr (str, '\\');
+	while (s != NULL) {
+		i++;
+		s++;
+		s = strchr (s, '\\');
+	}
+
+	s = (char *) malloc (str_len + i + 3);
+	if (s == NULL) {
+		return NULL;
+	}
+
+	i = 0;
+	if (add_quotes != 0) {
+		s[i++] = '"';
+	}
+
+	for (j = 0; j < str_len; j++) {
+		if (str[j] == '"' || str[j] == '\\') {
+			s[i++] = '\\';
+		}
+		s[i++] = str[j];
+	}
+	if (add_quotes != 0) {
+		s[i++] = '"';
+	}
+	s[i] = '\0';
+
+	return s;
+}
+
 /*------------------------======= End Of File =======------------------------*/
